########################################################################
# Module web2ldapcnf/hosts.py
# (c) by Michael Stroeder, michael.stroeder@inka.de
########################################################################
# This configuration module contains the pre-defined LDAP hosts
########################################################################

# List of all LDAP hosts to use. These hosts will appear in the
# default select list of the login form.
# A list containing only one host results in a normal input field
# with the host set as default.
ldap_host_list = [
                   'localhost',
                   'ldap.ms.inka.de',
                   'ldaptest.ms.inka.de',
                   'ldap.tu-chemnitz.de',
		   'nldap.com',
		   'ldap.novell.com',
		   'ldap.surfnet.nl',
		   'sun02.directory.dfn.de:17389',
		   'db.debian.org',
		   'elvira.innosoft.com'
		 ]


########################################################################
# LDAP host(s) with their defaults can be pre-defined as dictionary
# ldap_def = {'host:port':{paramdictionary}}
########################################################################

ldap_def = {
  'ldap.novell.com': {
    # Base DN for this host
    'basedn':'',
    # Template for bind-DN
    'binddn':r'',
    # Template for searching bind-DN
    'binddnsearch':r'(|(cn=*%s*)(mail=%s*))',
    # Regular expression for valid client cert DNs
    'sec_valid_dn':r'/.*',
    # Regular expression for valid issuer DNs of client certs
    'sec_valid_idn':r'/.*',
    # Required security level for each command {command:sec_ssllevel}
    # The sec_ssllevel is defined like this:
    # 0	no encryption at all
    # 1	session is encrypted with SSL and
    #       cipher is in sec_sslacceptedciphers (see below)
    # 2	client has valid certificate and
    # 	sec_valid_dn and sec_valid_idn are matched
    'sec_reqlevel': {
      'add':0,
      'delete':0,
      'modify':0,
      'passwd':0,
      'read':0,
      'vcard':0,
      'search':0,
      'print':0,
      'menu':0,
      'secinfo':0,
      'searchform':0,
      'filterform':0,
      'input':0
    },

    # LDAP-attributes which always should kept hidden from user.
    # This list should at least contain the attribute 'userPassword'!
    'hiddenattrs':['userpassword','modifytimestamp','modifiersname'],

    # These attributes with this order are listed in the table
    # of the search results. If set to empty list the DN is displayed.
    'search_tablistattrs':['givenname','sn','uid','mail','telephonenumber'],

    # This is the HTML template for the content of one table cell
    # Use a %(attr)s for each attribute.
    # (See Python's dictionary string formatting syntax)
    'print_template':r'<FONT SIZE=-3>%(givenname)s %(sn)s<br>Tel.: %(telephonenumber)s<br>FAX: %(facsimiletelephonenumber)s<br>E-Mail: %(mail)s</FONT>',
    # Number of columns for pretty-printable report
    'print_cols':4
  },

  'db.debian.org': {
    'basedn':'dc=debian,dc=org',
    'binddn':r'',
    'binddnsearch':r'',
    'sec_valid_dn':r'/.*',
    'sec_valid_idn':r'/.*',
    'sec_reqlevel':
      { 'add':0,'delete':0,'modify':0,'passwd':0,'read':0,'vcard':0,
	'search':0,'print':0,'menu':0,'secinfo':0,'searchform':0,
	'filterform':0,'input':0,'ldif':0
      },
    'search_tablistattrs':['cn','mail'],
    'print_template':r'<FONT SIZE=-3>%(givenname)s %(sn)s<br>Tel.: %(telephonenumber)s<br>FAX: %(facsimiletelephonenumber)s<br>E-Mail: %(mail)s</FONT>',
    'print_cols':4
  },

  'elvira.innosoft.com': {
    'basedn':'dc=Innosoft,dc=com',
    'sec_reqlevel':
      { 'add':0,'delete':0,'modify':0,'passwd':0,'read':0,'vcard':0,
	'search':0,'print':0,'menu':0,'secinfo':0,'searchform':0,
	'filterform':0,'input':0,'ldif':0
      },
    'search_tablistattrs':['cn','mail','homephone','telephonenumber'],
    'print_template':r'<FONT SIZE=-3>%(cn)s<br>%(streetaddress)s<br>%(l)s<br>%(homephone)s<br>%(telephonenumber)s<br>%(mobile)s<br>%(mail)s</FONT>',
    'print_cols':4
  },

  'openldap1-test': {
    'basedn':'dc=stroeder,dc=com',
    'binddn':r'mail=%(Email)s,o=%(O)s',
    'binddnsearch':r'(|(cn=*%s*)(mail=%s*)(uid=%s))',
    'passwd_hashtypes':['crypt','sSha'],
    'sec_valid_dn':r'/.*',
    'sec_valid_idn':r'/.*',
    'sec_reqlevel':
      { 'add':0,'delete':0,'modify':0,'passwd':0,'read':0,'vcard':0,
	'search':0,'print':0,'menu':0,'secinfo':0,'searchform':0,
	'filterform':0,'input':0,'ldif':0
      },
#    'hiddenattrs':['userpassword','modifiersname'],
    'search_tablistattrs':['cn','mail','homephone','telephonenumber'],
    'search_tdtemplate': {
      'organizationalUnit':r'Org. Unit &quot;%(ou)s&quot; Tel.: %(telephonenumber)s',
      'rfc822MailGroup':r'Mailing list %(cn)s &lt;%(mail)s&gt;, see %(labeledurl)s',
      'account':r'User account <strong>%(uid)s</strong>',
    },
    'print_template':r'<FONT SIZE=-3>%(cn)s<br>%(streetaddress)s<br>%(l)s<br>%(homephone)s<br>%(telephonenumber)s<br>%(mobile)s<br>%(mail)s</FONT>',
    'print_cols':4,
    'dit': {
      # 'DN':('DN component attribute of subordinate entries',[list of objectClasses],immutable)
      'dc=stroeder,dc=com':('ou',['organizationalUnit'],0),
      'ou=User,dc=stroeder,dc=com':('uid',['account','posixAccount'],0),
      'ou=mailto-tests,dc=stroeder,dc=com':('mail',['person','organizationalPerson','inetOrgPerson'],0)
    },
    'addform_template': {
      # 'object class':'pathname of HTML template file'
      'inetOrgPerson':'../templates/addform_inetOrgPerson.html',
      'posixAccount':'../templates/addform_posixAccount.html'
    },
    'read_template': {
      # 'object class':'pathname of HTML template file'
#      'inetOrgPerson':'/usr/lib/web2ldap/templates/read_inetOrgPerson.html',
#      'posixAccount':'/usr/lib/web2ldap/templates/read_posixAccount.html'
    }
  },

  # To all hacker wanna-be's:
  # This not a public accessible LDAP host. Don't try.
  # This meant as an more complex example section
  # Adjust the settings to reflect your local LDAP installation.
  'localhost': {
    'basedn':'o=Michaels',
    'binddn':r'mail=%(Email)s,o=%(O)s',
    'binddnsearch':r'(|(cn=*%s*)(mail=%s*)(uid=%s))',
    'sec_valid_dn':r'/.*',
    'sec_valid_idn':r'/.*',
    'sec_reqlevel': {
        'add':0,'delete':0,'modify':0,'passwd':0,'read':0,'vcard':0,
	'search':0,'print':0,'menu':0,'secinfo':0,'searchform':0,
	'filterform':0,'input':0,'ldif':0
      },
    'hiddenattrs':['userpassword','modifytimestamp','modifiersname'],
    'hiddenattrs':['modifytimestamp','modifiersname'],
    'search_tablistattrs':['cn','mail','homephone','telephonenumber'],
    'search_tdtemplate': {
      'organizationalUnit':r'Org. Unit &quot;%(ou)s&quot; Tel.: %(telephonenumber)s',
      'rfc822MailGroup':r'Mailing list %(cn)s &lt;%(mail)s&gt;, see %(labeledurl)s',
      'account':r'User account <strong>%(uid)s</strong>',
    },
    'print_template':r"""
<FONT SIZE=-3>
  %(cn)s<br>
  %(givenname)s %(sn)s<br>
  %(o)s<br>
  %(streetaddress)s<br>
  %(postalcode)s %(l)s<br>
  %(homephone)s<br>
  %(telephonenumber)s<br>
  %(facsimilenumber)s<br>
  %(mobile)s<br>
  %(mail)s
</FONT>""",
    'print_cols':4,
    'dit': {
      # 'DN':(
      #   'DN component attribute of subordinate entries',
      #   [list of objectClasses],
      #   immutable
      # )
      'o=Michaels':('ou',['organizationalUnit'],0),
      'ou=Bizness,o=Michaels':('cn',['person','organizationalPerson','inetOrgPerson'],0),
      'ou=Friends,o=Michaels':('cn',['person','organizationalPerson','inetOrgPerson'],0),
      'ou=Musicians,o=Michaels':('cn',['person','organizationalPerson','inetOrgPerson','musician'],0),
      'ou=User,o=Michaels':('uid',['account','posixAccount'],0),
      'ou=Internet Account Data,o=Michaels':('uid',['account','simpleSecurityObject'],0),
      'ou=Subscribed mailing lists,o=Michaels':('cn',['rfc822MailGroup'],0),
    },
    'searchform_template': '/usr/lib/web2ldap/templates/searchform_Base.html',
    'read_template': {
      # 'object class':'pathname of HTML template file'
#      'inetOrgPerson':'/usr/lib/web2ldap/templates/read_inetOrgPerson.html',
#      'posixAccount':'/usr/lib/web2ldap/templates/read_posixAccount.html',
#      'bankArrangement':'/usr/lib/web2ldap/templates/read_bankArrangement.html',
#      'musician':'/usr/lib/web2ldap/templates/read_musician.html',
    }
  },

  'ldaptest.ms.inka.de': {
    'basedn':'dc=stroeder,dc=com',
    'binddn':r'mail=%(Email)s,o=%(O)s',
    'binddnsearch':r'(|(cn=*%s*)(mail=%s*)(uid=%s))',
    'passwd_hashtypes':['crypt','sSha'],
    'sec_valid_dn':r'/.*',
    'sec_valid_idn':r'/.*',
    'sec_reqlevel':
      { 'add':0,'delete':0,'modify':0,'passwd':0,'read':0,'vcard':0,
	'search':0,'print':0,'menu':0,'secinfo':0,'searchform':0,
	'filterform':0,'input':0,'ldif':0
      },
#    'hiddenattrs':['userpassword','modifiersname'],
    'search_tablistattrs':['cn','mail','homephone','telephonenumber'],
    'search_tdtemplate': {
      'organizationalUnit':r'Org. Unit &quot;%(ou)s&quot; Tel.: %(telephonenumber)s',
      'rfc822MailGroup':r'Mailing list %(cn)s &lt;%(mail)s&gt;, see %(labeledurl)s',
      'account':r'User account <strong>%(uid)s</strong>',
    },
    'print_template':r'<FONT SIZE=-3>%(cn)s<br>%(streetaddress)s<br>%(l)s<br>%(homephone)s<br>%(telephonenumber)s<br>%(mobile)s<br>%(mail)s</FONT>',
    'print_cols':4,
    'dit': {
      # 'DN':('DN component attribute of subordinate entries',[list of objectClasses],immutable)
      'dc=stroeder,dc=com':('ou',['organizationalUnit'],0),
      'ou=User,dc=stroeder,dc=com':('uid',['account','posixAccount'],0),
      'ou=mailto-tests,dc=stroeder,dc=com':('mail',['person','organizationalPerson','inetOrgPerson'],0)
    },
    'addform_template': {
      # 'object class':'pathname of HTML template file'
      'inetOrgPerson':'../templates/addform_inetOrgPerson.html',
      'posixAccount':'../templates/addform_posixAccount.html'
    },
    'read_template': {
      # 'object class':'pathname of HTML template file'
#      'inetOrgPerson':'/usr/lib/web2ldap/templates/read_inetOrgPerson.html',
#      'posixAccount':'/usr/lib/web2ldap/templates/read_posixAccount.html'
    }
  }
}

