########################################################################
# Module web2ldapcnf/all.py
# (c) by Michael Stroeder, michael.stroeder@inka.de
########################################################################

# A list with directories containing additional Python modules
pylibdirs = ['/usr/local/web2ldap/pylib']

########################################################################
# HTTP-Options
########################################################################

# UTF-8 data is printed as is and charset is set to utf-8 in HTTP header:
# 0 Never
# 1 if the browser sends utf-8 in HTTP header and the web server sets
#   HTTP_ACCEPT_CHARSET environment variable
# 2 always
print_rawutf8=1

# Body can be sent to client gzipped if client presents
# gzip in HTTP_ACCEPT_ENCODING. You can set the gzip level (1-9) here.
# Set 0 turn it off. This needs zlibmodule which is automatically detected.
# Note: This might speed things up if the server's CPU is very fast
# compared to the network link. Even with large search results you
# won't gain much by choosing gzip level higher than 1.
gzip_level=0

########################################################################
# Other Options
########################################################################

# Maximum length of LDIF data in the <TEXTAREA> of addform/modifyform
ldif_maxbytes=200000

# Maximum count of input attribute fields in addform/modifyform
input_maxattrs=200

# Maximum length of attribute data input fields in addform/modifyform
input_maxfieldlen=300

# maximum count of search parameters in advanced search form
max_searchparams=9

# There are some situations where this client just wants to get the
# attributes of an entry and not the data itself for saving bandwidth.
# However some LDAP hosts (e.g. Notes Domino 4.61) have problems with
# such an attribute-only request, they won't return any matches for a search.
# If you experience this problems (no matching entry) set this to 0.
search_attrsonly=1

# Attributes which should be present in attribute select list of advanced search form
search_attrs = ['cn','mail','o','ou','st','uid','telephonenumber','description']

# Dictionary for defining user-friendly names of attributes types.
ldap_knownattr = {
  'c':'Country',
  'st':'State',
  'l':'Location',
  'o':'Organization',
  'ou':'Organizational Unit',
  'cn':'Common Name',
  'sn':'Surname',
  'givenname':'Given Name',
  'title':'Title',
  'streetaddress':'Street',
  'mail':'E-Mail',
  'uid':'User ID',
  'initials':'Initials',
  'objectclass':'Object Class',
  'telephonenumber':'Telephone',
  'facsimiletelephonenumber':'FAX',
  'homephone':'Home phone',
  'description':'Description'
}

# This sets the mapping of LDAP to vCard attributes
# See http://developer.netscape.com:80/docs/manuals/dsgw_html/dsgwparm.htm
vcard_property = {
  'fn':('cis',['cn']),
  'n':('cis',['sn','givenname']),
  'tel;pager':('cis',['mobile']),
  'tel;cell':('cis',['mobile']),
  'tel;fax':('cis',['facsimiletelephonenumber']),
  'tel;home':('cis',['homephone']),
  'tel;work':('cis',['telephonenumber']),
  'url':('cis',['labeleduri']),
  'org':('cis',['o','ou']),
  'email;internet':('cis',['mail']),
  'title':('cis',['title']),
  'role':('cis',['role']),
  'adr':('mls',['streetaddress','l','postalcode']),
  'note':('mls',['description'])
}

# map binary attributes to MIME-types
ldap_binaryattr = {
    'cacertificate;binary':('CA certificate','application/x-x509-ca-cert'),
    'certificaterevocationlist;binary':('User CRL','application/x-pkcs7-crl'),
    'certificate;binary':('Client certificate','application/x-x509-email-cert'),
    'usercertificate;binary':('Client certificate','application/x-x509-email-cert'),
    'usersmimecertificate;binary':('S/MIME certificate','application/x-x509-email-cert'),
    'certificate':('Client certificate','application/x-x509-email-cert'),
    'usercertificate':('Client certificate','application/x-x509-email-cert'),
    'usersmimecertificate':('S/MIME certificate','application/x-x509-email-cert'),
    'jpegphoto':('Photograph','image/jpeg'),
    'thumbnailphoto':('Photograph','image/jpeg'),
    'photo':('Photograph','image/jpeg'),
    'userpkcs12;binary':('PKCS#12','application/pkcs12'),
    'audio':('Audio','audio/basic')
  }

# Some browsers are more equal than others ;-)
ldap_browsermimetypes = {
    'certificaterevocationlist;binary': {
      'MSIE':'application/pkix-crl'
    }
  }

########################################################################
# HTML options
########################################################################

# Additional information in <HEAD></HEAD> section, e.g. CSS.
# Do not put <TITLE> tags in here!
#html_head = r"""
#  <link rel=stylesheet type="text/css" href="http://sites.inka.de/ms/css/web2ldap.css"/>
#"""
html_head = r"""
  <link rel=stylesheet type="text/css" href="/css/redandgreen.css"/>
"""

# Rudimental configuration for <BODY>-tag
#html_bodybegin = r"""
#<BODY LINK="Red" VLINK="Maroon" BGCOLOR="#fffffe">
#"""
html_bodybegin = r"""
<BODY>
"""

########################################################################
# Security options
########################################################################

# This is the HTTP method for submitting forms
# Use POST to avoid security problems with usernames and passwords
# to be stored in URLs (-> web server logs). Also POST can
# handle more input data.
# GET is quite handy for debugging and works reliable in stand-alone mode
# on all platforms.
#script_method = 'POST'
script_method = 'GET'

# You can set sec_expire to define the amount of
# time (in seconds) the data is valid in the browser
# (HTTP header Expires: is set).
# Note that if you're doing modifications you might get
# a very nasty behaviour if the pages are cached by your browser.
sec_expire = 0

# You can specify the required security level for each host and each command.
# This default is chosen if there's no definition for a specific
# host and command.
# Set this to 2 if you are using SSL web server with client certificates.
# Set to 1 if you use SSL connections.
# Set to 0 if all LDAP data you are handling through this gateway is public.
sec_reqlevel_default = 0

# Specifies a list of the acceptable symmetric key ciphers to
# reach security level 1.
# See also http://www.apache-ssl.org/docs.html and the ApacheSSL
# run-time directives SSLBanCipher, SSLRequireCipher, SSLRequiredCiphers
sec_sslacceptedciphers = [
# Strong ciphers
    'IDEA-CBC-SHA',
    'RC4-MD5',
    'RC4-SHA',
    'IDEA-CBC-MD5',
    'DES-CBC3-SHA',
    'DH-DSS-DES-CBC3-SHA',
    'DH-RSA-DES-CBC3-SHA',
    'EDH-DSS-DES-CBC3-SHA',
    'EDH-RSA-DES-CBC3-SHA',
    'ADH-RC4-MD5',
    'ADH-DES-CBC3-SHA',
    'FZA-RC4-SHA',
    'RC2-CBC-MD5',
    'DES-CBC3-MD5',
# Weak export ciphers
#    'EXP-RC4-MD5',
#    'EXP-RC2-CBC-MD5',
#    'EXP-DES-CBC-SHA',
#    'DES-CBC-SHA',
#    'EXP-DH-DSS-DES-CBC-SHA',
#    'DH-DSS-DES-CBC-SHA',
#    'EXP-DH-RSA-DES-CBC-SHA',
#    'DH-RSA-DES-CBC-SHA',
#    'EXP-EDH-DSS-DES-CBC-SHA',
#    'EDH-DSS-DES-CBC-SHA',
#    'EXP-EDH-RSA-DES-CBC',
#    'EDH-RSA-DES-CBC-SHA',
#    'EXP-ADH-RC4-MD5',
#    'EXP-ADH-DES-CBC-SHA',
#    'ADH-DES-CBC-SHA',
#    'DES-CFB-M1',
#    'DES-CBC-MD5',
#    'RC4-64-MD5'
  ]

