########################################################################
# charset.py Version 0.4.0
# (c) by Michael Stroeder <michael@stroeder.com>
########################################################################
# Module for converting characters sets
########################################################################

import sys, string

# Alphabet for encrypted passwords (see module crypt)
crypt_alphabet = './0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'
crypt_alphabet_len = len(crypt_alphabet)

known_charsets = ['ISO-8859-1','UTF-8','ASN.1','T.61','HTML4']

# returns 1 if s is plain ASCII
def is_ascii(s):
  if s:
    pos=0 ; s_len = len(s)
    while ((ord(s[pos]) & 0x80) == 0) and (pos<s_len-1):
      pos=pos+1
    if pos<s_len-1:
      return 0
    else:
      return (ord(s[pos]) & 0x80) == 0
  else:
    return 1

########################################################################
# Trys to escape all characters with a special meaning in HTML
# to appropriate character tags
########################################################################

def escapeHTML(s,escape_html_chars='&;<>":={}()'):
  for c in escape_html_chars:
    s=string.replace(s,c,'&#%d;'%ord(c))
  return s

########################################################################
# ISO-8859-1 in UTF-8 (Unicode) wandeln
########################################################################

def iso2utf(s):
  new = ''
  for ch in s:
    c=ord(ch)
    if (c & 0x80) == 0:
      new = new+ch
    else:
      new = new+chr(0xC0 | (0x03 & (c >> 6)))+chr(0x80 | (0x3F & c))
  return new

UTF8len= ( 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
	   1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
	   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	   2, 2, 2, 2, 2, 2, 2, 2, 3, 3, 3, 3, 4, 4, 5, 6 )

UTF8mask = (0x3F,0x7F,0x1F,0x0F,0x07,0x03,0x01)

########################################################################
# UTF-8 (Unicode) nach ISO-8859-1 wandeln
# nicht in ISO-8859-1 darstellbare Zeichen werden zu '?'
########################################################################

def utf2iso(s):

  new = ''
  ind = 0
  slen = len(s)

  while ind < slen:
    c=ord(s[ind])
    ind = ind+1
    clen = UTF8len[(c >> 2) & 0x3F]
    u = c & UTF8mask[clen]
    if clen==0:
      clen = 4
    else:
      clen = clen-1
    while clen and ind < slen:
      c=ord(s[ind])
      ind = ind+1
      if (c & 0xC0) == 0x80:
        u = (u << 6) | (c & 0x3F)
      else:
        ind = ind-1
        break
      clen = clen-1

    if (u <= 0xFF):
      new = new+chr(u)
    else:
      new = new+'?'

  return new

########################################################################
# UTF-8 (Unicode) in HTML 4 konforme Unicode-Darstellung wandeln
########################################################################

def utf2html4(s):

  new = ''
  ind = 0
  slen = len(s)

  while ind < slen:
    c=ord(s[ind])
    ind = ind+1
    clen = UTF8len[(c >> 2) & 0x3F]
    u = c & UTF8mask[clen]
    if clen==0:
      clen = 4
    else:
      clen = clen-1
    while clen and ind < slen:
      c=ord(s[ind])
      ind = ind+1
      if (c & 0xC0) == 0x80:
        u = (u << 6) | (c & 0x3F)
      else:
        ind = ind-1
        break
      clen = clen-1

    if u<128:
      new = new + chr(u)
    else:
      new = new + '&#%d;' % u

  return new

########################################################################
# ISO-Zeichen nach HTML 4 wandeln
########################################################################

def iso2html4(s):
  new = ''
  for ch in s:
    c=ord(ch)
    if (c & 0x80)==0:
      new = new + ch
    else:
      new = new + '&#%d;' % (c)
  return new

########################################################################
# ISO-8859-1 nach T61String wandeln
########################################################################

def iso2t61(s):
  new = ''
  for ch in s:
    c=ord(ch)
    if (c & 0x80) == 0:
      new = new+ch
    else:
      new = '%s\\x%X' % (new,ord(ch))
  return new

########################################################################
# T61String nach ISO-8859-1 wandeln
########################################################################

def t612iso(s):
  new = ''
  slashpos = string.find(s,'\\x')
  while slashpos!=-1:
    if (s[slashpos]==0) or (s[slashpos]>0 and s[slashpos-1]!='\\'):
      new = new+s[0:slashpos]+chr(string.atoi(s[slashpos+2:slashpos+4],16))
      s = s[slashpos+4:]
    else:
      new = new+s[0:slashpos-1]
      s = s[slashpos+1:]
    slashpos = string.find(s,'\\x')
  return new+s

########################################################################
# T61String nach HTML4 wandeln
########################################################################

def t612html4(s):
  new = ''
  slashpos = string.find(s,'\\x')
  while slashpos!=-1:
    if (s[slashpos]==0) or (s[slashpos]>0 and s[slashpos-1]!='\\'):
      new = new+s[0:slashpos]+'&#%d;' % string.atoi(s[slashpos+2:slashpos+4],16)
      s = s[slashpos+4:]
    else:
      new = new+s[0:slashpos-1]
      s = s[slashpos+1:]
    slashpos = string.find(s,'\\x')
  return new+s

########################################################################
# ISO-8859-1 nach ASN.1 wandeln
########################################################################

def iso2asn1(s):
  new = ''
  for ch in s:
    c=ord(ch)
    if (c & 0x80) == 0:
      new = '%s\\x00%s' % (new,ch)
    else:
      new = '%s\\x00\\x%X%s' % (new,ord(ch),ch)
  return new

########################################################################
# ASN.1 nach ISO-8859-1 wandeln
########################################################################

def asn12iso(s):
  return t612iso(string.replace(s,'\\x00',''))

########################################################################
# ASN.1 nach HTML4 wandeln
########################################################################

def asn12html4(s):
  return t612html4(string.replace(s,'\\x00',''))

########################################################################
# Allgemeine Recoding-Routine
########################################################################

recode_func = {
  'ISO-8859-1': {
      'UTF-8'      : iso2utf,
      'HTML4'      : iso2html4
    },
  'UTF-8': {
      'HTML4'      : utf2html4,
      'ISO-8859-1' : utf2iso
    }
  }

def recode(s,source,target):

  if source==target:
    return s

  return recode_func[string.upper(source)][string.upper(target)](s)

