##############################################################################
# dsml.py 0.1.0
# Various routines for handling DSML data
##############################################################################
# This module is distributed under the terms of the
# GPL (GNU GENERAL PUBLIC LICENSE) Version 2
# (see http://www.gnu.org/copyleft/gpl.html)
##############################################################################

import sys,string,StringIO,re,base64

dn_pattern   = r'([\w;.]+[\s]*=[^,]+)(,[ ]*[\w;.]+[\s]*=[^,]+)*'
dn_regex   = re.compile(dn_pattern)

# returns 1 if s is plain ASCII
def is_ascii(s):
  if s:
    pos=0 ; s_len = len(s)
    while ((ord(s[pos]) & 0x80) == 0) and (pos<s_len-1):
      pos=pos+1
    if pos<s_len-1:
      return 0
    else:
      return (ord(s[pos]) & 0x80) == 0
  else:
    return 1

########################################################################
# Write DSML formatted entry to outfile
# dn is the distinguished name as string
# entry is dictionary {attr:data}
########################################################################

def CreateDSML(outfile,dn,entry={},binary_attrs=[]):

  # Write line dn: first
  outfile.write('<dsml:entry dn="%s">\n' % (dn))

  objectclasses = entry.get('objectclass',entry.get('objectClass',[]))

  outfile.write('<dsml:objectclass>\n')
  for oc in objectclasses:
    outfile.write('<dsml:oc-value>%s</dsml:oc-value>\n' % oc)
  outfile.write('</dsml:objectclass>\n')

  attrs = entry.keys()[:]
  try:
    attrs.remove('objectclass')
    attrs.remove('objectClass')
  except:
    pass
  attrs.sort()
  for attr in attrs:
    outfile.write('<dsml:attr name="%s">\n' % (attr))
    for data in entry[attr]:
      if attr in binary_attrs:
  	outfile.write('<dsml:value encoding="base64">\n%s</dsml:value>\n' % (base64.encodestring(data)))
      else:
  	outfile.write('<dsml:value>%s</dsml:value>\n' % (data))
    outfile.write('</dsml:attr>\n')
  outfile.write('</dsml:entry>\n')
  return

