##############################################################################
# ldapbase.py Version 0.1.6
# (c) by Michael Stroeder <michael@stroeder.com>
##############################################################################
# This module is distributed under the terms of the
# GPL (GNU GENERAL PUBLIC LICENSE) Version 2
# (see http://www.gnu.org/copyleft/gpl.html)
########################################################################

import sys, string, re, UserDict, ldap

ldap_mod_str = {ldap.MOD_ADD:'add',ldap.MOD_REPLACE:'replace',ldap.MOD_DELETE:'delete'}
ldap_searchscope_str = ['base','one','sub']
ldap_searchscope = {
  'base':ldap.SCOPE_BASE,
  'one':ldap.SCOPE_ONELEVEL,
  'sub':ldap.SCOPE_SUBTREE
}

#dn_pattern   = r'([\w;.]+[\s]*=[^,]+)(,[ ]*[\w;.]+[\s]*=[^,]+)*'

attr_pattern = r'[\w;.]+(;[\w_-]+)*'
data_pattern = '([^,]+|".*?")'
rdn_pattern = attr_pattern + r'[\s]*=[\s]*' + data_pattern
dn_pattern   = rdn_pattern + r'([\s]*,[\s]*' + rdn_pattern + r')*'

#rdn_regex   = re.compile('^%s$' % rdn_pattern)
dn_regex   = re.compile('^%s$' % dn_pattern)

# returns 1 if s is a LDAP DN
def is_dn(s):
  rm = dn_regex.match(s)
  return rm!=None

def normalize_dn(dn):
  result = string.split(dn,',')
  result = map(string.strip,result)
  return string.join(result,',')

# returns parent-DN of dn
def ParentDN(dn):
  return string.join(string.split(dn,',')[1:],',')

# returns parent-DN of dn
def SplitRDN(dn):
  try:
    rdn,basedn = string.split(dn,',',1)
    return (rdn,basedn)
  except ValueError:
    return (dn,'')

# returns a list of parent-DNs of dn
def ParentDNList(dn):
  result = []
  DNComponentList = string.split(dn,',')
  for i in range(1,len(DNComponentList)):
    result.append(string.join(DNComponentList[i:],','))
  return result

# parse a LDAP URL and return (host,dn,attributes,scope,filter)
# host         LDAP host
# dn           distinguished name
# attributes   list with attributes
# scope        search scope string
# filter       LDAP search filter
def parse_ldap_url(ldap_url):
  dummy,rest = string.split(ldap_url,'://',1)
  try:
    host,rest = string.split(rest,'/',1)
  except ValueError:
    host='' ; dn=rest
  paramlist=string.split(rest,'?')
  dn          = paramlist[0]
  try:
    attributes  = string.split(paramlist[1],',')
  except IndexError:
    attributes  = []
  try:
    scope       = paramlist[2]
  except IndexError:
    scope       = ''
  try:
    filter      = paramlist[3]
  except IndexError:
    filter      = ''
  return (host,dn,attributes,scope,filter)


# Build modify list for call of method add()
def add_modifylist(entry):

  modlist = []
  for attr in entry.keys():
    modlist.append((attr,entry[attr]))
  return modlist


# Build differential modify list for call of method modify()
def modify_modifylist(old_entry,new_entry):

  modlist = []
  new_entry_keys = new_entry.keys()
  old_entry_keys = old_entry.keys()
  for attr in new_entry.keys():
    if string.lower(attr) in old_entry_keys:
      attr_lower = string.lower(attr)
      old_data = old_entry.get(attr,old_entry.get(attr_lower,[]))
      if new_entry[attr]!=old_data:
	if old_data and (new_entry[attr]==[]) or (new_entry[attr]==['']):
	  # delete an existing attribute
	  modlist.append((ldap.MOD_DELETE,attr,old_data))
	else:
	  # modify an existing attribute
	  modlist.append((ldap.MOD_REPLACE,attr,new_entry[attr]))
    else:
      for data in new_entry[attr]:
	if data:
	  # neue Attribute hinzufuegen
	  modlist.append((ldap.MOD_ADD,attr,data))

  return modlist


