########################################################################
# web2ldap
# (c) by Michael Stroeder <michael@stroeder.com>
# Distributed under GNU Public License (GPL)
# web-based LDAP Client, see http://www.web2ldap.de for details
########################################################################

import sys, string, \
       charset, cgiforms, ldap, ldapbase, w2lcore, web2ldapcnf
from msbase import listremove

########################################################################
# Add new entry
########################################################################

def w2l_Add(outf,command,form,ldap_session):

  def AddListTable(form,modlist):
    stringlist = []
    stringlist.append('<TABLE BORDER RULES=rows CELLSPACING=0 CELLPADDING=2%%>')
    for i in modlist:
      if i[0] in w2lcore.ldap_binaryattrkeys:
        tablestr = '%s Bytes of binary data' % string.join(map(str,map(len,i[1])),'Bytes, ')
      else:
        tablestr = string.join(w2lcore.utf2display(form.accept_charset,i[1]),'<BR>')
      stringlist.append('<TR><TD>%s</TD><TD>%s</TD></TR>' % (
        w2lcore.AttrNameStr(i[0]),
	tablestr
      )
    )
    stringlist.append('</TABLE>')
    return string.join(stringlist,'\n')

  ldap_host,ldap_binddn,ldap_bindpasswd,ldap_obj,ldap_defaults,ldap_dn = ldap_session

  if 'ldap_oc' in form.inputkeys:
    objectclasses = form.field['ldap_oc'][0].content
  else:
    raise w2lcore.ErrorExitClass(ldap_session,'No object class(es) defined.')

  new_rdn_attr,default_objectclasses,dit_immutable = ldap_defaults.get('dit',{}).get(ldap_dn,('',[],0))

  if 'input_ldif' in form.inputkeys:
    from StringIO import StringIO
    import ldif
    f = StringIO(form.field['input_ldif'][0].content)
    try:
      ldif_data = ldif.ParseLDIF(f,maxentries=1)
    except ValueError:
      raise w2lcore.ErrorExitClass(ldap_session,'Error parsing LDIF data.')

    entry = ldif_data[0][1]
    if entry.has_key('objectclass'):
      entry['objectclass'].extend(objectclasses)
    else:
      entry['objectClass'] = objectclasses
  else:
    entry = {'objectClass':objectclasses}

  if 'input_attr' in form.inputkeys:
    for i in range(len(form.field['input_attr'][0].content)):
      attr = form.field['input_attr'][0].content[i]
      if attr:
	data = w2lcore.input2utf(form.accept_charset,form.field['input_data'][0].content[i])
	if data:
	  if entry.has_key(attr):
	    entry[attr].append(data)
	  else:
	    entry[attr] = [data]

  if not dit_immutable and 'ldap_rdn' in form.inputkeys:
    ldap_rdn = w2lcore.input2utf(form.accept_charset,form.field['ldap_rdn'][0].content)
  else:
    if entry.has_key(new_rdn_attr):
      ldap_rdn='%s=%s' % (new_rdn_attr,entry[new_rdn_attr][0])
    else:
      raise w2lcore.ErrorExitClass(ldap_session,'Required <STRONG>attribute</STRONG> %s not in entry data.' % (new_rdn_attr))

  if not ldap_rdn:
    raise w2lcore.ErrorExitClass(ldap_session,'Unable to determine RDN for new entry.')

  if ldap_dn:
    ldap_dn = string.join([ldap_rdn,ldap_dn],',')
  else:
    ldap_dn = ldap_rdn

  # Modify-Liste zumsammenbauen
  modlist = ldapbase.add_modifylist(entry)

  if not modlist:
    raise w2lcore.ErrorExitClass(ldap_session,'Cannot add entry without attributes.')

  # Neuen Eintrag zum Verzeichnis schicken
  try:
    ldap_obj.add_s(ldap_dn,modlist)
  except ldap.NO_SUCH_OBJECT:
    parent_dn = ldapbase.ParentDN(ldap_dn)
    missing_parentdns = []
    basedn = charset.iso2utf(ldap_defaults.get('basedn',''))
    while parent_dn!=basedn:
      try:
        parent_entry = ldap_obj.search_s(parent_dn,ldap.SCOPE_BASE,"objectclass=*",['objectclass'],web2ldapcnf.search_attrsonly)[0][1].keys()
      except ldap.NO_SUCH_OBJECT:
        missing_parentdns.append(parent_dn)
      parent_dn = ldapbase.ParentDN(parent_dn)
    if missing_parentdns:
      w2lcore.PrintHeader(outf,'Error',form.accept_charset)
      w2lcore.MainMenuTable(outf,form,ldap_session)
      outf.write("""<DIV ID=MessageDiv><H1>Error</H1>
<P>No such object: The entry <STRONG>%s</STRONG> was not correctly added.</P>
Missing parent entries:
<TABLE>""" % (w2lcore.utf2display(form.accept_charset,ldap_dn)))
      for parent_dn in missing_parentdns:
        rdn,dn = ldapbase.SplitRDN(parent_dn)
        outf.write('<TR><TD>%s</TD><TD>%s</TD></TR>' % (
	  parent_dn,
	  w2lcore.CommandButton(form,'addform','Create entry',w2lcore.utf2display(form.accept_charset,ldap_binddn),w2lcore.utf2display(form.accept_charset,ldap_bindpasswd),ldap_host,w2lcore.utf2display(form.accept_charset,dn),extrastr='<INPUT TYPE=HIDDEN NAME="ldap_rdn" VALUE="%s">'%(w2lcore.utf2display(form.accept_charset,rdn)),target='_inputform'),
	))
      outf.write('</TABLE></DIV>')
      w2lcore.PrintFooter(outf)
      return

    else:
      add_ldap_dn = ldap_dn
      ldap_dn = ldapbase.ParentDN(ldap_dn)
      raise w2lcore.ErrorExitClass(ldap_session,"""No such object: The entry %s was not correctly added.<BR>Most likely this is because of missing parent entries.""" % w2lcore.utf2display(form.accept_charset,add_ldap_dn))

  except ldap.ALREADY_EXISTS:
    raise w2lcore.ErrorExitClass(ldap_session,'Entry <STRONG>%s</STRONG> already exists. You probably did not define a new (relative) DN.' % (w2lcore.utf2display(form.accept_charset,ldap_dn)))
  except ldap.OBJECT_CLASS_VIOLATION:
    raise w2lcore.ErrorExitClass(ldap_session,'Object class violation: Maybe a required attribute is missing or some of the given attributes are not allowed.')
  except ldap.INSUFFICIENT_ACCESS:
    raise w2lcore.ErrorExitClass(ldap_session,'You are not allowed to add the new entry <STRONG>%s</STRONG>.' % (w2lcore.utf2display(form.accept_charset,ldap_dn)))
  except:
    exc_obj,exc_value,exc_traceback = sys.exc_info()
    raise w2lcore.ErrorExitClass(ldap_session,'Unhandled exception: %s, %s.' % (exc_obj,exc_value))

  w2lcore.PrintHeader(outf,'',form.accept_charset)
  w2lcore.MainMenuTable(outf,form,ldap_session)
  w2lcore.CommandTable(outf,[
    w2lcore.CommandButton(form,'vcard','vCard',w2lcore.utf2display(form.accept_charset,ldap_binddn),w2lcore.utf2display(form.accept_charset,ldap_bindpasswd),ldap_host,w2lcore.utf2display(form.accept_charset,ldap_dn),target='_labeledurl'),
    w2lcore.CommandButton(form,'ldif','LDIF',w2lcore.utf2display(form.accept_charset,ldap_binddn),w2lcore.utf2display(form.accept_charset,ldap_bindpasswd),ldap_host,w2lcore.utf2display(form.accept_charset,ldap_dn),
	          extrastr='<INPUT TYPE=HIDDEN NAME="search_scope" VALUE="base"><INPUT TYPE=HIDDEN NAME="search_filterstr" VALUE="(objectClass=*)">',target='_altformat'),
    w2lcore.CommandButton(form,'dsml','DSML',w2lcore.utf2display(form.accept_charset,ldap_binddn),w2lcore.utf2display(form.accept_charset,ldap_bindpasswd),ldap_host,w2lcore.utf2display(form.accept_charset,ldap_dn),
	          extrastr='<INPUT TYPE=HIDDEN NAME="search_scope" VALUE="base"><INPUT TYPE=HIDDEN NAME="search_filterstr" VALUE="(objectClass=*)">',target='_altformat'),
    w2lcore.CommandButton(form,'modifyform','Modify',w2lcore.utf2display(form.accept_charset,ldap_binddn),w2lcore.utf2display(form.accept_charset,ldap_bindpasswd),ldap_host,w2lcore.utf2display(form.accept_charset,ldap_dn)),
    w2lcore.CommandButton(form,'modrdn','Modify RDN',w2lcore.utf2display(form.accept_charset,ldap_binddn),w2lcore.utf2display(form.accept_charset,ldap_bindpasswd),ldap_host,w2lcore.utf2display(form.accept_charset,ldap_dn)),
    w2lcore.CommandButton(form,'delete','Delete',w2lcore.utf2display(form.accept_charset,ldap_binddn),w2lcore.utf2display(form.accept_charset,ldap_bindpasswd),ldap_host,w2lcore.utf2display(form.accept_charset,ldap_dn)),
    w2lcore.CommandButton(form,'passwd','Password of entry',w2lcore.utf2display(form.accept_charset,ldap_binddn),w2lcore.utf2display(form.accept_charset,ldap_bindpasswd),ldap_host,w2lcore.utf2display(form.accept_charset,ldap_dn))
  ],div_id='ContextMenuDiv',table_id='ContextMenuTable')
  outf.write('<DIV ID=MessageDiv><P>Added entry <STRONG>%s</STRONG> with your input data:</P>\n%s\n</DIV>' % (w2lcore.utf2display(form.accept_charset,ldap_dn),AddListTable(form,modlist)))

  w2lcore.PrintFooter(outf)


