########################################################################
# web2ldap
# (c) by Michael Stroeder <michael@stroeder.com>
# Distributed under GNU Public License (GPL)
# web-based LDAP Client, see http://www.web2ldap.de for details
########################################################################

import sys, string, re, \
       cgiforms, ldap, w2lcore, web2ldapcnf, msbase
from ldapbase import ParentDN
from msbase import union,listremove

########################################################################
# Input form for add and modify
########################################################################

def w2l_AddModifyForm(outf,command,form,ldap_session):

  def PrintObjectClassForm(outf,form,command,ldap_dn,ldap_binddn,ldap_bindpasswd,ldap_host,ldap_rdn):

    outf.write('<div id=InputDiv>')

    if command=='addform':
      outf.write('<h1>Add new entry</h1>\nChoose object class(es) for new entry.')

    elif command=='modifyform':
      outf.write('<h1>Modify entry</h1>\nYou may change the object class(es) for the entry.')

    if w2lcore.utf2display(form.accept_charset,ldap_rdn):
      ldap_rdn_input = '<input type=hidden name="ldap_rdn" value="%s">' % w2lcore.utf2display(form.accept_charset,ldap_rdn)
    else:
      ldap_rdn_input = ''

    outf.write("""
    <FORM ACTION="%s/%s" METHOD="%s" ENCTYPE="application/x-www-form-urlencoded" ACCEPT-CHARSET="%s">
    <input type=hidden name="ldap_dn"     value="%s">
    %s
    <input type=hidden name="ldap_binddn"     value="%s">
    <input type=hidden name="ldap_bindpasswd" value="%s">
    <input type=hidden name="ldap_host"       value="%s">
    <input type="submit" value="Input form">
    <p>%s</P>
    </FORM></div>
    """ % ( \
      form.script_name,command,
      form.request_method,form.accept_charset,
      w2lcore.utf2display(form.accept_charset,ldap_dn),
      ldap_rdn_input,
      w2lcore.utf2display(form.accept_charset,ldap_binddn),
      w2lcore.utf2display(form.accept_charset,ldap_bindpasswd),
      ldap_host,
      form.field['ldap_oc'][0].inputfield()
    ))

  def PrintInputTable(outf,form,attrs,tableheader='',entry={}):
    outf.write('<tr><TH COLSPAN=2>%s</TH></tr>' % tableheader)
    for attr in attrs:
      datalist = entry.get(string.lower(attr),entry.get(attr,['']))
      for data in datalist:
	outf.write("""<tr>
      <td>%s<input type=hidden name="input_attr" value="%s"></td>
      <td><input name="input_data" value="%s" size=50></td>
    </tr>
  """ % (w2lcore.AttrNameStr(attr),attr,w2lcore.utf2display(form.accept_charset,data)))

  from ldapoc import ldap_oc
  known_objectclasses = ldap_oc.keys()
  known_objectclasses.sort()

  ldap_host,ldap_binddn,ldap_bindpasswd,ldap_obj,ldap_defaults,ldap_dn = ldap_session

  entry ={}

  if command=='addform':
    if 'ldap_rdn' in form.inputkeys:
      ldap_rdn = w2lcore.input2utf(form.accept_charset,form.field['ldap_rdn'][0].content)
    else:
      ldap_rdn = ''
    new_rdn_attr,objectclasses,dit_immutable = ldap_defaults.get('dit',{}).get(ldap_dn,('',[],0))
    form.field['ldap_oc'][0].setdefault(objectclasses)
  if command=='modifyform':
    ldap_rdn = ''
    if not ldap_dn:
      raise w2lcore.ErrorExitClass(ldap_session,'The DN of the entry to be modified is needed.')
    new_rdn_attr,objectclasses,dit_immutable = ldap_defaults.get('dit',{}).get(ParentDN(ldap_dn),('',[],0))

  if not dit_immutable:
    if not 'ldap_oc' in form.inputkeys:
      # Print form for selecting object class(es)
      if command=='modifyform':
	# Erst mal nur alle Attribute des Eintrags lesen
	try:
	  result_dnlist = ldap_obj.search_s(ldap_dn,ldap.SCOPE_BASE,"objectclass=*",['objectclass'],0)
	  entry = result_dnlist[0][1]
	  existing_objectclasses = entry.get('objectclass',entry.get('objectClass',[]))
	except ldap.NO_SUCH_OBJECT:
	  raise w2lcore.ErrorExitClass(ldap_session,'No entry <STRONG>%s</STRONG>.' % (w2lcore.utf2display(form.accept_charset,ldap_dn)))
	except:
	  exc_obj,exc_value,exc_traceback = sys.exc_info()
          raise w2lcore.ErrorExitClass(ldap_session,'Unhandled exception: %s, %s.' % (exc_obj,exc_value))
        # Add content of current objectClass attribute to
	# select list for choosing objectClass'es
        form.field['ldap_oc'][0].options = union(existing_objectclasses,form.field['ldap_oc'][0].options)
	form.field['ldap_oc'][0].options.sort()
	form.field['ldap_oc'][0].setdefault(existing_objectclasses)

      w2lcore.PrintHeader(outf,'',form.accept_charset)
      w2lcore.MainMenuTable(outf,form,ldap_session)
      PrintObjectClassForm(outf,form,command,w2lcore.utf2display(form.accept_charset,ldap_dn),w2lcore.utf2display(form.accept_charset,ldap_binddn),w2lcore.utf2display(form.accept_charset,ldap_bindpasswd),ldap_host,w2lcore.utf2display(form.accept_charset,ldap_rdn))
      w2lcore.PrintFooter(outf)
      return

    else:
      # Read objectclass(es) from input form
      objectclasses = form.field['ldap_oc'][0].content

  if command=='modifyform':

    # Erst mal nur alle Attribute des Eintrags lesen
    try:
      # Get all attribute names of entries
      existing_attrs = ldap_obj.search_s(ldap_dn,ldap.SCOPE_BASE,"objectclass=*",None,web2ldapcnf.search_attrsonly)[0][1].keys()
      # Falls nicht nur ein Attributinhalt gelesen werden soll,
      # so werden alle Binaerattribute, fuer die ein
      # MIME-Typ-Mapping angegeben wurde, rausgestrichen
      read_attrs = existing_attrs[:]
      listremove(read_attrs,w2lcore.ldap_binaryattrkeys,1)
      # Read all non-binary attributes of entry
      result_dnlist = ldap_obj.search_s(ldap_dn,ldap.SCOPE_BASE,"objectclass=*",read_attrs,0)
      entry = result_dnlist[0][1]
    except ldap.NO_SUCH_OBJECT:
      raise w2lcore.ErrorExitClass(ldap_session,'No entry <STRONG>%s</STRONG>.' % (w2lcore.utf2display(form.accept_charset,ldap_dn)))
    except:
      exc_obj,exc_value,exc_traceback = sys.exc_info()
      raise w2lcore.ErrorExitClass(ldap_session,'Unhandled exception: %s, %s.' % (exc_obj,exc_value))

  ActionStr = '%s%s' % (string.upper(command[0]),command[1:-4])

  required_attrs = []
  allowed_attrs = []
  # Make a case-ignoring string-keyed copy of ldap_oc
  ldap_oc_ci = msbase.CaseinsensitiveStringKeyDict(ldap_oc)

  for oc in objectclasses:
    oc_def = ldap_oc_ci.get(oc,{})
    required_attrs = union(required_attrs,oc_def.get('requires',[]))
    allowed_attrs = union(allowed_attrs,oc_def.get('allows',[]))

  donottouch_attrs = ['objectclass']+ldap_defaults.get('hiddenattrs',[])
  listremove(required_attrs,donottouch_attrs,1)
  listremove(required_attrs,w2lcore.ldap_binaryattrkeys,1)
  listremove(allowed_attrs,donottouch_attrs,1)
  listremove(allowed_attrs,w2lcore.ldap_binaryattrkeys,1)
  listremove(allowed_attrs,required_attrs,1)

  w2lcore.PrintHeader(outf,'',form.accept_charset)
  w2lcore.MainMenuTable(outf,form,ldap_session)
  outf.write("""<div id=InputDiv>
<h1>%s entry</h1>
<form action="%s/%s" method="%s" enctype="application/x-www-form-urlencoded" accept-charset="%s">
""" % (ActionStr,form.script_name,command[:-4],form.request_method,form.accept_charset))

  if command=='addform':
    if ldap_rdn or (not new_rdn_attr or not dit_immutable):
      outf.write('<p>RDN for new entry below <STRONG>%s</STRONG>:<br><INPUT NAME="ldap_rdn" value="%s" SIZE=50></P>' % (w2lcore.utf2display(form.accept_charset,ldap_dn),w2lcore.utf2display(form.accept_charset,ldap_rdn)))
  outf.write('<input type=hidden name="ldap_dn" SIZE=50 value="%s">' % (w2lcore.utf2display(form.accept_charset,ldap_dn)))

  # Print input form according to object class(es)
  outf.write("""
  <input type=hidden name="ldap_binddn"     value="%s">
  <input type=hidden name="ldap_bindpasswd" value="%s">
  <input type=hidden name="ldap_host"       value="%s">
  <input type="submit" value="%s"> <input type="reset" value="Reset">
""" % ( \
    w2lcore.utf2display(form.accept_charset,ldap_binddn),
    w2lcore.utf2display(form.accept_charset,ldap_bindpasswd),
    ldap_host,
    ActionStr,
  ))

  # Print chosen objectclass(es) as hidden input
  for oc in objectclasses:
    outf.write('<input type=hidden name="ldap_oc" value="%s">' % (oc))

  outf.write("""
<table id=InputTable summary="Input form for LDAP entry data" border rules=rows cellspacing="2%%" cellpadding="2%%">
<tr>
  <td>%s</td>
  <td>%s</td>
</tr>
"""% ( \
    w2lcore.AttrNameStr('objectclass'),
    string.join(objectclasses,'<br>')
  ))

  PrintInputTable(outf,form,required_attrs,'Required attributes',entry)
  PrintInputTable(outf,form,allowed_attrs,'Allowed attributes',entry)
  if command=='modifyform':
    # Print attributes which does not appear in the schema but
    # do exist in the entry
    listremove(read_attrs,['objectclass','modifytimestamp','creatorsname','createtimestamp','modifiersname','userpassword'],1)
    listremove(read_attrs,required_attrs,1)
    listremove(read_attrs,allowed_attrs,1)
    if read_attrs:
      PrintInputTable(outf,form,read_attrs,'Existing attributes not matching schema',entry)

  outf.write("""
<tr>
  <td align=left valign=top>
    <p><em>Raw LDIF data</EM></p>
    Notes:<br>
    <ul>
      <li>Lines containing "dn:" will be ignored</LI>
      <li>Only the first entry (until first empty line) will be accepted</LI>
      <li>NON-ASCII characters are not translated</LI>
      <li>maximum length is set to %d bytes
    </ul>
  </td>
  <td>
    <textarea name="input_ldif" value="" wrap=off rows=10 cols=70></textarea>
  </td>
</tr>
</table>
</form>
</div>
""" % (web2ldapcnf.ldif_maxbytes))

  w2lcore.PrintFooter(outf)

