########################################################################
# web2ldap
# (c) by Michael Stroeder <michael@stroeder.com>
# Distributed under GNU Public License (GPL)
# web-based LDAP Client, see http://www.web2ldap.de for details
########################################################################

Version = '0.7.9'

import sys, os, string, types, time, re, \
       cgiforms, ldap, ldapbase, cgissl, charset, web2ldapcnf, msgzip

from ldapbase import ParentDN,ParentDNList,is_dn,normalize_dn,dn_pattern

def utf2display(dispcharset,data):
  if type(data) is types.StringType:
    return charset.escapeHTML(charset.recode(data,'utf-8',dispcharset),'<>"(){}')
  elif type(data) is types.ListType:
    l = []
    for i in data:
      l.append(utf2display(dispcharset,i))
    return l
  else:
    raise TypeError, "expected string or list"

def input2utf(inputcharset,s):
  return charset.recode(s,inputcharset,'utf-8')

########################################################################
# Initialize some constants used throughout the whole script
########################################################################

host_pattern = '([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+|[a-zA-Z]+[a-zA-Z0-9-]*(\.[a-zA-Z]+[a-zA-Z0-9-]*)*)?(:[0-9]*|)?'
url_pattern  = r'^(ftp|http|https|news|ldap|ldaps)?:(|//)?[^ ]*'
url_regex  = re.compile(url_pattern)
labeleduri_regex = re.compile(url_pattern+r' .*')
timestamp_pattern = r'^([0-9]){12,14}Z$'
timestamp_regex  = re.compile(timestamp_pattern)
mail_pattern = r'^([\w@.=/_ -]+)@([\w-]+)(\.[\w-]+)*$'
mail_regex = re.compile(mail_pattern)
ldap_binaryattrkeys = web2ldapcnf.ldap_binaryattr.keys()

# Define a attribute name <-> user friendly attribute name mapping
ldap_knownattr = web2ldapcnf.ldap_knownattr
for i in ldap_binaryattrkeys:
  ldap_knownattr[i] = web2ldapcnf.ldap_binaryattr[i][0]
ldap_knownattrkeys = ldap_knownattr.keys()
ldap_knownattrkeys.sort()

search_option = {
  r'(%s=*%s*)':'contains',
  r'(!(%s=*%s*))':"doesn't contain",
  r'(%s=%s)':'is',
  r'(!(%s=%s))':'is not',
  r'(%s=%s*)':'begins with',
  r'(%s=*%s)':'ends with',
  r'(%s~=%s)':'sounds like'
}

class ErrorExitClass:

  def __init__(self,ldap_session,Msg):
    self.ldap_session = ldap_session
    self.Msg = Msg

########################################################################
# haeufig gebrauchte Funktionen
########################################################################

# This function searches for variants
def GetVariantFilename(pathname,variantlist):
  for v in variantlist:
    variant_filename = pathname+'.'+v
    if os.path.isfile(variant_filename):
      break
  else:
    variant_filename = pathname
    if not os.path.isfile(variant_filename):
      variant_filename = ''
  return variant_filename


def CommandButton(form,command,submitstr,binddn='',bindpasswd='',host='',dn='',extrastr='',target='_top'):
  if dn:
    dnstr = '<input type=hidden name="ldap_dn" value="%s">' % (dn)
  else:
    dnstr = ''
  return """
<form
  id=CommandForm
  action="%s/%s"
  method="%s"
  target="%s"
  enctype="application/x-www-form-urlencoded"
  accept-charset="%s"
>
<input type=hidden name="ldap_binddn"     value="%s">
<input type=hidden name="ldap_bindpasswd" value="%s">
<input type=hidden name="ldap_host"       value="%s">
%s
<input id=CommandButton type="submit" value="%s">
%s
</form>""" % (
      form.script_name,
      command,
      web2ldapcnf.script_method,
      target,
      form.accept_charset,
      binddn,
      bindpasswd,
      host,
      dnstr,
      submitstr,
      extrastr
    )


# Print a table containing CommandButton()s
def CommandTable(outf,commandlist,div_id='CommandDiv',table_id='CommandTable'):
  outf.write('<div id="%s"><table id="%s" summary="Command buttons" cellpadding="0%%" cellspacing="0%%"><tr>' % (div_id,table_id))
  for s in commandlist:
    outf.write('<td id=MenuItem align=center valign=top>%s</td>' % s)
  outf.write('</tr></table></div>')


# Print the main menu at top of screen
def MainMenuTable(outf,form,ldap_session,current_dn=''):

  CommandList = []
  ldap_host,ldap_binddn,ldap_bindpasswd,ldap_obj,ldap_defaults,ldap_dn = ldap_session

  # convert and store to speed things up
  ldap_binddn_disp     = utf2display(form.accept_charset,ldap_binddn)
  ldap_bindpasswd_disp = utf2display(form.accept_charset,ldap_bindpasswd)
  ldap_dn_disp         = utf2display(form.accept_charset,ldap_dn)

  if current_dn:
    ldap_dn = current_dn

  if ldap_host:

    CommandList.append(CommandButton(form,'search','Go down',ldap_binddn_disp,ldap_bindpasswd_disp,ldap_host,ldap_dn_disp,extrastr='<INPUT TYPE=HIDDEN NAME="search_scope" VALUE="one"><INPUT TYPE=HIDDEN NAME="search_filterstr" VALUE="(objectclass=*)">'))

    ldap_basedn = ldap_defaults.get('basedn','')
    if ldap_dn!=ldap_basedn:
      parent_dn_list = string.split(ldap_dn_disp,',')
      parent_dn_len = len(parent_dn_list)-1
      parent_dn_select = []
      i = 0
      dn = ldap_dn
      while (i<parent_dn_len) and (dn!=ldap_basedn):
	i=i+1
	dn = string.join(parent_dn_list[i:],',')
	parent_dn_select.append('<option value="%s">%s</option>' % (dn,parent_dn_list[i]))
      if not ldap_basedn:
	parent_dn_select.append('<option value="">Root DSE</option>')

      CommandList.append(CommandButton(form,'search','Go up',ldap_binddn_disp,ldap_bindpasswd_disp,ldap_host,'',extrastr='<br><FONT SIZE=-2><SELECT NAME="ldap_dn">%s</SELECT></FONT><INPUT TYPE=HIDDEN NAME="search_scope" VALUE="one"><INPUT TYPE=HIDDEN NAME="search_filterstr" VALUE="(objectclass=*)">' % string.join(parent_dn_select,'\n')))

    if ldap_basedn:
      search_dn = ldap_basedn
    else:
      search_dn = ldap_dn
    CommandList.append(
      CommandButton(form,'searchform','Search',
	ldap_binddn_disp,
  	ldap_bindpasswd_disp,ldap_host,utf2display(form.accept_charset,search_dn),
	extrastr='<br><select name="searchform_mode"><option value="base">Base</option><option value="adv">Advanced</option><option value="exp">Expert</option></select>'
      )
    )

    CommandList.extend(
      [
        CommandButton(form,'read','Read',ldap_binddn_disp,ldap_bindpasswd_disp,ldap_host,ldap_dn_disp),
	CommandButton(form,'addform','New entry',ldap_binddn_disp,ldap_bindpasswd_disp,ldap_host,ldap_dn_disp),
      ]
    )

    if ldap_binddn:
      CommandList.append(
        CommandButton(form,'passwd','Password',ldap_binddn_disp,ldap_bindpasswd_disp,ldap_host,ldap_binddn_disp)
      )

    CommandList.append(
      CommandButton(form,'secinfo','Security',ldap_binddn_disp,ldap_bindpasswd_disp,ldap_host,ldap_dn_disp)
    )

  CommandList.append(CommandButton(form,'login','Login','','',ldap_host))

  outf.write('<div id=MainMenuDiv>')
  CommandTable(outf,CommandList,table_id='MainMenuTable')
  outf.write('<p><strong>%s</strong> on <strong>%s</strong>' % (
    DataStr(form,ldap_session,ldap_dn)+"Root DSE"*(ldap_dn_disp==''),
    ldap_host
  ))
  if ldap_binddn:
    outf.write(', bound as <strong>%s</strong>' % (ldap_binddn_disp))
  outf.write('</p></div>')


# Return a binddn formed by a template and
# a subject DN of a client certificate
def CreateDefaultBindDN(binddn_template,clientcertdn):

  def Splitclientcertdn(DN):
    result = {}
    s = string.split(DN[1:],'/')
    for i in s:
      try:
	id,value = string.split(i,'=',1)
	result[id] = value
      except:
	pass
    return result

  clientcertdnlist = Splitclientcertdn(clientcertdn)
  try:
    return charset.iso2utf(charset.asn12iso(binddn_template % clientcertdnlist))
  except KeyError:
    return ''


# Return a pretty HTML-formatted string for describing the attribute
def AttrNameStr(attr):
  attr = string.lower(attr)
  if attr in ldap_knownattrkeys:
    return '<strong>%s</strong><br><em>(%s)</em>' % (ldap_knownattr[attr],attr)
  else:
    return '<em>%s</em>' % attr


# Return a pretty HTML-formatted string of the attribute data
def DataStr(form,ldap_session,data,attr='',commandbutton=0):

  ldap_host,ldap_binddn,ldap_bindpasswd,ldap_obj,ldap_defaults,ldap_dn = ldap_session

#  try:
  if ldap.is_ldap_url(data):
    ldap_host,ldap_dn,search_attr,search_scope,search_filterstr = ldapbase.parse_ldap_url(data)
    if search_filterstr:
      commandbuttonstr = \
        CommandButton(
	  form,
	  'search',
	  'Search',
	  utf2display(form.accept_charset,ldap_binddn),
	  utf2display(form.accept_charset,ldap_bindpasswd),
	  ldap_host,
	  utf2display(form.accept_charset,ldap_dn),
	  extrastr='<input type=hidden name="search_scope" value="%s"><input type=hidden name="search_filterstr" value="%s">' % (search_scope,search_filterstr)
	)
    else:
      commandbuttonstr = \
        CommandButton(
	  form,
	  'read',
	  'Read',
	  utf2display(form.accept_charset,ldap_binddn),
	  utf2display(form.accept_charset,ldap_bindpasswd),
	  ldap_host,
	  utf2display(form.accept_charset,ldap_dn)
	)

    return '<table><tr><td>%s</td><td><a href="%s" target="_ldapurl">%s</a></td></tr></table>' % (
             commandbuttonstr,
	     charset.utf2iso(data),
	     utf2display(form.accept_charset,data)
	   )
#  except TypeError:
#    pass

  rm = mail_regex.match(data)
  if rm!=None:
    return '<a href="mailto:%s">%s</a>' % (data,data)

  if is_dn(data):
    href_str = '<a href="ldap://%s/%s" target="_ldapurl">%s</a>' % (
             ldap_host,
	     charset.utf2iso(data),
	     utf2display(form.accept_charset,data)
	   )
    if commandbutton:
      return '<table><tr><td>%s</td><td>%s</td></tr></table>' % (
	       CommandButton(form,'read','Read',utf2display(form.accept_charset,ldap_binddn),utf2display(form.accept_charset,ldap_bindpasswd),ldap_host,utf2display(form.accept_charset,data)),
	       href_str
	     )
    else:
      return href_str

  rm = url_regex.search(data)
  if rm!=None:
    rm = labeleduri_regex.match(data)
    if rm!=None:
      url,label = string.split(data,' ',1)
      return '<a href="%s" target="_labeledurl">%s</a>' % (url,utf2display(form.accept_charset,label))
    else:
      return '<a href="%s" target="_labeledurl">%s</a>' % (data,data)

  rm = timestamp_regex.match(data)
  if rm!=None:
    if len(data)==15:
      return '%s-%s-%sT%s:%s:%s-0000' % (data[0:4],data[4:6],data[6:8],data[8:10],data[10:12],data[12:14])
    elif len(data)==13:
      # old format without century
      year = int(data[0:2])
      if year<=50:
        return '20%s-%s-%sT%s:%s:%s-0000' % (data[0:2],data[2:4],data[4:6],data[6:8],data[8:10],data[10:12])
      else:
        return '19%s-%s-%sT%s:%s:%s-0000' % (data[0:2],data[2:4],data[4:6],data[6:8],data[8:10],data[10:12])
    else:
      return data
  return string.join(string.split(utf2display(form.accept_charset,data),'$'),'<br>')


########################################################################
# Funktionen fuer immer wiederkehrende HTML-Ausgaben
########################################################################


# Write HTTP-Header
def HTTPHeader(
  outf=sys.stdout,
  contenttype='text/html',
  charset='ISO-8859-1',
  contentlength='',
  expires_offset='',
  lastmodified='',
  additional_header = {}
):

  # Return seconds as RFC1123 Date/Time Format preferred
  # in RFC2616 (HTTP 1.1)
  def DateTimeRFC1123(secs=0):
    return time.strftime(
      '%a, %d %b %Y %H:%M:%S GMT',
      time.gmtime(secs)
    )

  gzip = hasattr(outf,'fileobj')
  if gzip:
    headerf = outf.fileobj
  else:
    headerf = outf
 
  # Determine times
  gmt = time.time()
  if not lastmodified:
    lastmodified = DateTimeRFC1123(gmt)
  if not expires_offset:
    expires = DateTimeRFC1123(gmt)
  else:
    expires = DateTimeRFC1123(gmt+expires_offset)

  # Write header
  headerf.write('Content-Type: %s;charset=%s\r\n' % (contenttype,charset))
  if contentlength and not gzip:
    headerf.write('Content-Length: %s\r\n' % (contentlength))
  if gzip:
    headerf.write('Content-Encoding: gzip\r\n')
    headerf.write('Vary: Accept-Encoding\r\n')

  headerf.write('Last-Modified: %s\r\n' % (lastmodified))
  headerf.write('Expires: %s\r\n' % (expires))
  for h in additional_header.keys():
    headerf.write('%s: %s\r\n' % (h,additional_header[h]))
  # Write empty end-of-header line
  headerf.write('\r\n')
  headerf.flush()
  return


# Ausdrucken eines HTML-Kopfes mit Titelzeile
def PrintHeader(outf,TitleMsg,HTTP_charset='ISO-8859-1'):
  
  HTTPHeader(
    outf,
    'text/html',
    HTTP_charset,
    expires_offset=web2ldapcnf.sec_expire,
  )

  outf.write("""<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">
<html>
<head>
  <meta name="generator" content="web2ldap %s, see www.web2ldap.de"/>
  <title>web2ldap %s:</title>
  <meta http-equiv="Content-Type" content="text/html;charset=%s"></meta>
  %s
</head>
%s""" % (
    Version,
    TitleMsg,
    HTTP_charset,
    web2ldapcnf.html_head,
    web2ldapcnf.html_bodybegin
  ))
  return


# Ausdrucken eines HTML-Endes
def PrintFooter(f):
  f.write("""<div id=AboutDiv>
  <p align=center>
    <font size=-2>
      Powered by
      <a href="http://www.web2ldap.de/" target="_web2ldap">web2ldap %s</a>
    </font>
  </p>
</div>
</body>
</html>""" % (Version))
  return

def LDAPSessionParams(command,form):

  if 'ldap_host' in form.inputkeys:
    ldap_host = form.field['ldap_host'][0].content
  else:
    ldap_host = ''
  if 'ldap_dn' in form.inputkeys:
    ldap_dn = input2utf(form.accept_charset,form.field['ldap_dn'][0].content)
  else:
    ldap_dn = ''

  if not ldap_host:
    return ('','','',None,{},'')

  ldap_defaults = web2ldapcnf.ldap_def.get(ldap_host,{})

  ldap_basedn = charset.iso2utf(ldap_defaults.get('basedn',''))

  try:
    ldap_obj = ldap.open(ldap_host)
  except:
    ldap_host = ''
    exc_obj,exc_value,exc_traceback = sys.exc_info()
    raise ErrorExitClass((ldap_host,'','',None,{},''),'Unhandled exception during connecting to %s:<p>%s, %s</p>' % (ldap_host,exc_obj,exc_value))

  if 'ldap_bindpasswd' in form.inputkeys:

    ldap_bindpasswd = input2utf(form.accept_charset,form.field['ldap_bindpasswd'][0].content)

    if 'ldap_binddn' in form.inputkeys:

      ldap_binddn = input2utf(form.accept_charset,form.field['ldap_binddn'][0].content)

      if not is_dn(ldap_binddn):
	# Do a smart login by searching unique entry
	ldap_searchdnfilter = string.replace(ldap_defaults.get('binddnsearch',''),'%s',ldap_binddn)
	try:
	  result = ldap_obj.search_s(ldap_basedn,ldap.SCOPE_SUBTREE,ldap_searchdnfilter,['objectclass'],web2ldapcnf.search_attrsonly)
	except ldap.NO_SUCH_OBJECT:
	  raise ErrorExitClass(
	    (ldap_host,'','',None,{},''),
	    'Smart login did not find any entry for user name <strong>%s</strong>.' % (ldap_binddn)
	  )
	except ldap.FILTER_ERROR:
	  raise ErrorExitClass(
	    (ldap_host,'','',None,{},''),
	    'Bad search filter for smart login.<br>This is likely a configuration problem.'
	  )
	except:
	  exc_obj,exc_value,exc_traceback = sys.exc_info()
	  raise ErrorExitClass(
	    ('','','',None,{},''),
	    'Unhandled exception during smart login on %s:<p>%s, %s.</p>' % (ldap_host,exc_obj,exc_value)
	  )
	if result:
          if len(result)==1:
            ldap_binddn = result[0][0]
	  else:
    	    raise ErrorExitClass(
	      (ldap_host,'','',None,{},''),
	      'Smart login did not find a unique entry to authenticate for user name.<br>It might help to provide a more detailed user name.'
	    )
	else:
	  raise ErrorExitClass(
	    (ldap_host,'','',None,{},''),
	    'Smart login did not find any entry for user name.'
	  )
    else:
      ldap_binddn = CreateDefaultBindDN(ldap_defaults.get('binddn',''),form.env.get('SSL_CLIENT_DN',''))

    ldap_binddn = normalize_dn(ldap_binddn)

  else:
    ldap_binddn = ''
    ldap_bindpasswd = ''

  ldap_dn = normalize_dn(ldap_dn)

  try:
    ldap_obj.bind_s(ldap_binddn,ldap_bindpasswd,ldap.AUTH_SIMPLE)
  except ldap.INVALID_CREDENTIALS:
    raise ErrorExitClass(('','','',None,{},''),'Wrong password for binding as <strong>%s</strong> to %s.' % (utf2display(form.accept_charset,ldap_binddn),ldap_host))
  except ldap.INAPPROPRIATE_AUTH:
    raise ErrorExitClass(('','','',None,{},''),'Wrong password for binding as <strong>%s</strong> to %s.' % (utf2display(form.accept_charset,ldap_binddn),ldap_host))
  except:
    exc_obj,exc_value,exc_traceback = sys.exc_info()
    raise ErrorExitClass((ldap_host,'','',None,{},''),'Unhandled exception during binding as <strong>%s</strong> to %s:<p>%s, %s.</p>' % (utf2display(form.accept_charset,ldap_binddn),ldap_host,exc_obj,exc_value))

  return (ldap_host,ldap_binddn,ldap_bindpasswd,ldap_obj,ldap_defaults,ldap_dn)


def CheckSecurityLevel(command,ldap_session):
  ldap_host,ldap_binddn,ldap_bindpasswd,ldap_obj,ldap_defaults,ldap_dn = ldap_session
  sec_valid_dn = ldap_defaults.get('sec_valid_dn','')
  sec_valid_idn = ldap_defaults.get('sec_valid_idn','')
  sec_ssllevel = cgissl.SecLevel(
    web2ldapcnf.sec_sslacceptedciphers,
    sec_valid_dn,
    sec_valid_idn
  )
  # Ist der Sicherheitslevel ausreichend fuer das geforderte Kommando?
  sec_reqlevel = ldap_defaults.get('sec_reqlevel',{}).get(command,web2ldapcnf.sec_reqlevel_default)
  if sec_ssllevel < sec_reqlevel:
    raise ErrorExitClass(ldap_session,'Security level %d required for executing command %s on host %s! The security level of your access was only %d.' % (sec_reqlevel,command,ldap_host,sec_ssllevel)
    )
  return
