########################################################################
# web2ldap
# (c) by Michael Stroeder <michael@stroeder.com>
# Distributed under GNU Public License (GPL)
# web-based LDAP Client, see http://www.web2ldap.de for details
########################################################################

import sys,cgiforms, ldap, ldapbase, w2lcore, web2ldapcnf
from ldapbase import ParentDN

########################################################################
# Delete entry
########################################################################

def w2l_Delete(outf,command,form,ldap_session):

  ldap_host,ldap_binddn,ldap_bindpasswd,ldap_obj,ldap_defaults,ldap_dn = ldap_session

  if not ldap_dn:
    raise w2lcore.ErrorExitClass(ldap_session,'No DN to delete.')

  if 'delete_confirm' in form.inputkeys:

    if form.field['delete_confirm'][0].content=='yes':

      # Recursive delete of whole sub-tree

      if ('delete_recursive' in form.inputkeys) and \
         form.field['delete_recursive'][0].content=='yes':

        def ContinueErrorExit(outf,continueonerror,Msg):
	  outf.write('%s<BR>' % Msg)

        def DelTree(DelTree,outf,form,ContinueErrorExit,ldap_obj,dn,continueonerror=0):

          outf.write('<DL><DT>Searching entries below <STRONG>%s</STRONG>...</DT>' % (w2lcore.utf2display(form.accept_charset,ldapbase.SplitRDN(dn)[0])))
          try:
            result = ldap_obj.search_s(dn,ldap.SCOPE_ONELEVEL,'(objectclass=*)',['objectclass'],web2ldapcnf.search_attrsonly)
	  except ldap.NO_SUCH_OBJECT:
	    result = []
            ContinueErrorExit(outf,continueonerror,'No entry <STRONG>%s</STRONG>.' % (w2lcore.utf2display(form.accept_charset,ldapbase.SplitRDN(dn)[0])))
	  except:
	    result = []
            exc_obj,exc_value,exc_traceback = sys.exc_info()
            ContinueErrorExit(outf,continueonerror,'Unhandled exception: %s, %s.' % (exc_obj,exc_value))
          outf.write('<DD>')
          for entry in result:
	    DelTree(DelTree,outf,form,ContinueErrorExit,ldap_obj,entry[0],continueonerror)
	  try:
	    ldap_obj.delete_s(dn)
            outf.write('Deleted entry <STRONG>%s</STRONG>.<BR>' % (w2lcore.utf2display(form.accept_charset,ldapbase.SplitRDN(dn)[0])))
	  except ldap.INSUFFICIENT_ACCESS:
  	    ContinueErrorExit(outf,continueonerror,'You are not allowed to delete entry <STRONG>%s</STRONG>.' % (w2lcore.utf2display(form.accept_charset,ldapbase.SplitRDN(dn)[0])))
	  except ldap.NOT_ALLOWED_ON_NONLEAF:
	    ContinueErrorExit(outf,continueonerror,'Entry <STRONG>%s</STRONG> still has sub-entries.' % (w2lcore.utf2display(form.accept_charset,ldapbase.SplitRDN(dn)[0])))
	  except:
	    exc_obj,exc_value,exc_traceback = sys.exc_info()
  	    ContinueErrorExit(outf,continueonerror,'Unhandled exception: %s, %s.</P>' % (exc_obj,exc_value))
          outf.write('</DD></DL>')

        continue_on_error = ('delete_coe' in form.inputkeys) and \
                            form.field['delete_coe'][0].content=='yes'

	w2lcore.PrintHeader(outf,'Delete tree',form.accept_charset)
	w2lcore.MainMenuTable(outf,form,ldap_session,ParentDN(ldap_dn))
        outf.write('<DIV ID=MessageDiv><P>Delete tree below entry <STRONG>%s</STRONG>.</P>' % (w2lcore.utf2display(form.accept_charset,ldap_dn)))
	DelTree(DelTree,outf,form,ContinueErrorExit,ldap_obj,ldap_dn,continue_on_error)

      else:

	try:
	  ldap_obj.delete_s(ldap_dn)
	except ldap.NO_SUCH_OBJECT:
	  raise w2lcore.ErrorExitClass(ldap_session,'No entry <STRONG>%s</STRONG>.' % (w2lcore.utf2display(form.accept_charset,ldap_dn)))
	except ldap.INSUFFICIENT_ACCESS:
	  raise w2lcore.ErrorExitClass(ldap_session,'You are not allowed to delete entry <STRONG>%s</STRONG>.' % (w2lcore.utf2display(form.accept_charset,ldap_dn)))
	except ldap.NOT_ALLOWED_ON_NONLEAF:
	  raise w2lcore.ErrorExitClass(ldap_session,'Entry <STRONG>%s</STRONG> has sub-entries. Try recursive delete.' % (w2lcore.utf2display(form.accept_charset,ldap_dn)))
	except:
	  exc_obj,exc_value,exc_traceback = sys.exc_info()
	  raise w2lcore.ErrorExitClass(ldap_session,'Unhandled exception: %s, %s.' % (exc_obj,exc_value))

	w2lcore.PrintHeader(outf,'Deleted Entry',form.accept_charset)
	w2lcore.MainMenuTable(outf,form,ldap_session,ParentDN(ldap_dn))
        outf.write('<DIV ID=MessageID><P>Deleted entry <STRONG>%s</STRONG>.</P>' % (w2lcore.utf2display(form.accept_charset,ldap_dn)))

      outf.write('</P></DIV>')
      w2lcore.PrintFooter(outf)

    else:
      raise w2lcore.ErrorExitClass(ldap_session,'Canceled delete.')

  else:

    # Print confirmation form
    w2lcore.PrintHeader(outf,'Delete entry?',form.accept_charset)
    w2lcore.MainMenuTable(outf,form,ldap_session)
    outf.write("""<DIV ID=MessageDiv>
<FORM ACTION="%s/delete"
      METHOD="%s"
      ENCTYPE="application/x-www-form-urlencoded"
      ACCEPT-CHARSET="%s"
>
  <INPUT TYPE=HIDDEN NAME="ldap_dn" VALUE="%s">
  <INPUT TYPE=HIDDEN NAME="ldap_host" VALUE="%s">
  <INPUT TYPE=HIDDEN NAME="ldap_binddn" VALUE="%s">
  <INPUT TYPE=HIDDEN NAME="ldap_bindpasswd" VALUE="%s">
  <P>Delete <STRONG>%s</STRONG>. Are you sure?</P>
  <P>
    %s Recursive delete of all sub-entries. <STRONG>Use with care!</STRONG><BR>
    %s Continue on error (affects only recursive delete).
  </P>
  <INPUT NAME="delete_confirm" TYPE="submit" VALUE="yes">
  <INPUT NAME="delete_confirm" TYPE="submit" VALUE="no">
</FORM>
</DIV>
""" % (
      form.script_name,
      web2ldapcnf.script_method,
      form.accept_charset,
      w2lcore.utf2display(form.accept_charset,ldap_dn),
      ldap_host,
      w2lcore.utf2display(form.accept_charset,ldap_binddn),
      w2lcore.utf2display(form.accept_charset,ldap_bindpasswd),
      w2lcore.utf2display(form.accept_charset,ldap_dn),
      form.field['delete_recursive'][0].inputfield(),
      form.field['delete_coe'][0].inputfield()
    ))
    w2lcore.PrintFooter(outf)

