########################################################################
# web2ldap
# (c) by Michael Stroeder <michael@stroeder.com>
# Distributed under GNU Public License (GPL)
# web-based LDAP Client, see http://www.web2ldap.de for details
########################################################################

import sys, os, string, re, ldap, cgiforms, ldif, ldapbase, web2ldapcnf, w2lcore, msgzip

def HandleHTTPRequest(
  inf=sys.stdin,
  outfile=sys.stdout,
  env=os.environ
):

  if ('gzip' in string.split(string.lower(env.get('HTTP_ACCEPT_ENCODING','')),',')) and \
     (web2ldapcnf.gzip_level>0) and \
     msgzip.GzipFile:
    # Create new gzip-stream file object
    outf = msgzip.GzipFile(
      filename='web2ldap output',
      mode='wb',
      compresslevel=web2ldapcnf.gzip_level,
      fileobj=outfile
    )
  else:
    outf = outfile

  http_accept_charset = env.get('HTTP_ACCEPT_CHARSET')
  if http_accept_charset and web2ldapcnf.print_rawutf8:
    if ('utf-8' in string.split(http_accept_charset,',')) or \
       (web2ldapcnf.print_rawutf8==2):
      accept_charset = 'utf-8'
    else:
      accept_charset = 'iso-8859-1'
  else:
    accept_charset = 'iso-8859-1'

  # form initialisieren
  form = cgiforms.formClass(inf,env,accept_charset)

  # Diese Felder werden eigentlich von fast allen Kommandos gebraucht
  form.add(cgiforms.formInputClass('ldap_sessionid','LDAP Session ID',255,'.*',size=25))
  form.add(cgiforms.formInputClass('ldap_binddn','Bind DN',255,'.*',size=40))
  form.add(cgiforms.formInputClass('ldap_bindpasswd','with Password',30,'.*',size=15))
  form.add(cgiforms.formInputClass('ldap_host','Host:Port',255,w2lcore.host_pattern,size=30))
  form.add(cgiforms.formInputClass('ldap_dn','DN',255,ldapbase.dn_pattern,size=25))

  command = env.get('PATH_INFO','/login')[1:]
  if not command:
    command = 'login'

  if command=='searchform':
    form.add(cgiforms.formSelectClass('searchform_mode','Search form mode',[('base','Base'),('adv','Advanced'),('exp','Expert')],'base'))
    form.add(cgiforms.formFieldClass('search_params','Anzahl Suchangaben',2,'[0-9]+'))
    form.add(cgiforms.formSelectClass('search_scope','Scope',[('base','Base'),('one','One level'),('sub','Sub tree')],'sub'))
    form.add(cgiforms.formInputClass('search_filterstr','Search filter string',120,'.*'))
    form.add(cgiforms.formSelectClass('search_resnumber','Number of results to display',[('0','unlimited'),('10','10'),('20','20'),('50','50'),('100','100'),('200','200')],'10'))

  elif command=='search':
    # Felder definieren
    form.add(cgiforms.formInputClass('search_filterstr','Search filter string',120,'.*'))
    form.add(cgiforms.formInputClass('search_resminindex','Minimum index of search results',10,'[0-9]+'))
    form.add(cgiforms.formInputClass('search_resnumber','Number of results to display',3,'[0-9]+'))
    form.add(cgiforms.formSelectClass('search_output','Search output format',['table','ldif','ldif1','dsml','print']))
    form.add(cgiforms.formSelectClass('search_mode','Search Mode',[r'(&%s)',r'(|%s)']))
    form.add(cgiforms.formSelectClass('search_scope','Scope',ldapbase.ldap_searchscope_str))
    form.add(cgiforms.formInputClass('search_attr','Search attribute',250,'[\w]+',multiple=web2ldapcnf.max_searchparams))
    form.add(cgiforms.formSelectClass('search_option','Search option',w2lcore.search_option.keys(),multiple=web2ldapcnf.max_searchparams))
    form.add(cgiforms.formInputClass('search_string','Search string',60,'.*',multiple=web2ldapcnf.max_searchparams))

  elif command in ['addform','modifyform']:
    from ldapoc import ldap_oc
    known_objectclasses = ldap_oc.keys()
    known_objectclasses.sort()
    if command=='addform':
      form.add(cgiforms.formInputClass('ldap_rdn','RDN of new entry',255,'.*'))
    form.add(cgiforms.formSelectClass('ldap_oc','Object classes',known_objectclasses,size=12,multiple=1))

  elif command=='add':
    from ldapoc import ldap_oc
    known_objectclasses = ldap_oc.keys()
    form.add(cgiforms.formInputClass('ldap_rdn','RDN of new entry',255,'.*'))
    form.add(cgiforms.formSelectClass('ldap_oc','Object classes',known_objectclasses,size=10,multiple=1))
    form.add(cgiforms.formTextareaClass('input_ldif','LDIF data',web2ldapcnf.ldif_maxbytes,ldif.ldif_pattern))
    form.add(cgiforms.formInputClass('input_attr','Attributename',255,'[a-zA-Z]+[a-zA-Z0-9;]*',multiple=web2ldapcnf.input_maxattrs))
    form.add(cgiforms.formInputClass('input_data','Attributedata',web2ldapcnf.input_maxfieldlen,'.*',multiple=web2ldapcnf.input_maxattrs))

  elif command=='modify':
    from ldapoc import ldap_oc
    known_objectclasses = ldap_oc.keys()
    form.add(cgiforms.formSelectClass('ldap_oc','Object classes',known_objectclasses,size=10,multiple=1))
    # FIX ME!!! This UGLY UGLY UGLY UGLY UGLY!
    form.field['ldap_oc'][0].regex = re.compile(r'[\w]*')
    form.add(cgiforms.formInputClass('input_attr','Attribute Name',255,'[a-zA-Z]+[a-zA-Z0-9;]*',multiple=web2ldapcnf.input_maxattrs))
    form.add(cgiforms.formInputClass('input_data','Attribute Data',web2ldapcnf.input_maxfieldlen,'.*',multiple=web2ldapcnf.input_maxattrs))
  #  form.add(cgiforms.formTextareaClass('input_ldif','LDIF data',web2ldapcnf.ldif_maxbytes,r'(^(\w+(:|::) |\s+).*$\n)*'))
    form.add(cgiforms.formTextareaClass('input_ldif','LDIF data',web2ldapcnf.ldif_maxbytes,r'.*'))

  elif command=='delete':
    form.add(cgiforms.formSelectClass('delete_confirm','Confirmation',['yes','no']))
    form.add(cgiforms.formCheckboxClass('delete_recursive','recursive delete','yes',checked=0))
    form.add(cgiforms.formCheckboxClass('delete_coe','continue on error','yes',checked=0))

  elif command=='modrdn':
    form.add(cgiforms.formInputClass('modrdn_newrdn','New RDN',255,ldapbase.dn_pattern,size=50))
    form.add(cgiforms.formCheckboxClass('modrdn_delold','Delete old',value="yes",checked=1))

  elif command=='passwd':
    import w2lpasswd
    form.add(cgiforms.formFieldClass('passwd_newpasswd','New password',100,'.*',multiple=2))
    form.add(cgiforms.formSelectClass('passwd_hash','Hash',w2lpasswd.available_hashtypes,w2lpasswd.available_hashtypes[-1]))

  elif command=='read' or command=='vcard':
    form.add(cgiforms.formInputClass('read_attr','Read attribute',255,'[a-zA-Z]+[a-zA-Z0-9;]*'))
    form.add(cgiforms.formInputClass('read_attrindex','Read attribute',255,'[0-9]+'))

  elif command=='secinfo':
    pass

  elif command=='login':
    form.add(cgiforms.formInputClass('login_binddn','Bind DN',255,'.*',size=25))

  try:

    ldap_session = ('','','',None,{},'')

    try:
      form.getparams(ignoreemptyparams=1)

    # catch exceptions of parameter parsing
    except cgiforms.formContentLengthException,e:
      raise w2lcore.ErrorExitClass(ldap_session,'Content length invalid. Received %d bytes (max. &d allowed).' % (e.contentlength,e.maxcontentlength))
    except cgiforms.formParamNameException,e:
      ldap_session = w2lcore.LDAPSessionParams(command,form)
      raise w2lcore.ErrorExitClass(ldap_session,'Unknown parameter <strong>%s</strong>.' % (e.name))
    except cgiforms.formParamContentException,e:
      ldap_session = w2lcore.LDAPSessionParams(command,form)
      raise w2lcore.ErrorExitClass(ldap_session,'Content of field <strong>%s</strong> has invalid format.<br>Input was: &quot;%s&quot;' % (e.text,e.content))
    except cgiforms.formParamStructException,e:
      ldap_session = w2lcore.LDAPSessionParams(command,form)
      raise w2lcore.ErrorExitClass(ldap_session,'Too many (%d) parameters for field <strong>%s</strong>.' % (e.count,e.name))
    except cgiforms.formParamLengthException,e:
      ldap_session = w2lcore.LDAPSessionParams(command,form)
      raise w2lcore.ErrorExitClass(ldap_session,'Content too long. Field <strong>%s</strong> has %d characters but is limited to %d.' % (e.text,e.length,e.maxlength))

    ldap_session = w2lcore.LDAPSessionParams(command,form)
    w2lcore.CheckSecurityLevel(command,ldap_session)

    if command=='searchform':
      import w2lsearchform
      w2lsearchform.w2l_SearchForm(outf,command,form,ldap_session)

    elif command=='search':
      import w2lsearch
      w2lsearch.w2l_Search(outf,command,form,ldap_session)

    elif command=='addform' or command=='modifyform':
      import w2laddmodifyform
      w2laddmodifyform.w2l_AddModifyForm(outf,command,form,ldap_session)

    elif command=='add':
      import w2ladd
      w2ladd.w2l_Add(outf,command,form,ldap_session)

    elif command=='modify':
      import w2lmodify
      w2lmodify.w2l_Modify(outf,command,form,ldap_session)

    elif command=='delete':
      import w2ldelete
      w2ldelete.w2l_Delete(outf,command,form,ldap_session)

    elif command=='modrdn':
      import w2lmodrdn
      w2lmodrdn.w2l_ModRDN(outf,command,form,ldap_session)

    elif command=='passwd':
      import w2lpasswd
      w2lpasswd.w2l_Passwd(outf,command,form,ldap_session)

    elif command=='read' or command=='vcard':
      import w2lread
      w2lread.w2l_Read(outf,command,form,ldap_session)

    elif command=='secinfo':
      import w2lsecinfo
      w2lsecinfo.w2l_SecInfo(outf,command,form,ldap_session)

    elif command=='login':
      import w2llogin
      w2llogin.w2l_Login(outf,command,form,ldap_session)

    else:
      ldap_session = ('','','','','','')
      raise w2lcore.ErrorExitClass(ldap_session,'Invalid web2ldap command <STRONG>%s</STRONG>!' % (command))

  except ldap.error,e:

    w2lcore.PrintHeader(outf,'LDAP Error',form.accept_charset)
    w2lcore.MainMenuTable(outf,form,e.ldap_session)
    outf.write("""<div id=MessageDiv>
<h1>LDAP Error</h1>
The underlying LDAP module raised an exception not
explicitly handled by web2ldap.
<dl>
    <dt>%s</dt>
    <dd>%s</dd>
</dl>
For questions or error reports contact:
<a href="mailto:%s">%s</a>
</div>
""" % (e['info'],e['description'],env.get('SERVER_ADMIN',''),env.get('SERVER_ADMIN','')))
    w2lcore.PrintFooter(outf)
    # help garbage collection
    sys.exc_traceback = None
    e = None

  except w2lcore.ErrorExitClass,e:

    w2lcore.PrintHeader(outf,'Error',form.accept_charset)
    w2lcore.MainMenuTable(outf,form,e.ldap_session)
    outf.write("""<div id=MessageDiv>
<h1>Error</h1>
<p>
  %s
</p>
For questions or error reports contact:
<a href="mailto:%s">%s</a>
</div>
""" % (e.Msg,env.get('SERVER_ADMIN',''),env.get('SERVER_ADMIN','')))
    w2lcore.PrintFooter(outf)
    # help garbage collection
    sys.exc_traceback = None
    e = None

  # Close gzip file-object if necessary
  if hasattr(outf,'fileobj'):
    outf.close()
    outf = None

  # Close LDAP connection object if necessary
  l = ldap_session[3]
  if l:
    l.unbind_s()
    l = None

  return

