########################################################################
# web2ldap
# (c) by Michael Stroeder <michael@stroeder.com>
# Distributed under GNU Public License (GPL)
# web-based LDAP Client, see http://www.web2ldap.de for details
########################################################################

import sys,string,charset,web2ldapcnf,cgiforms,ldap,w2lcore

# Try to determine the avaiable hash types
available_hashtypes = [('','None')]

# Check if all required modules are in place to
# determine the available encryption methods
try:
  import random
  import base64
  import crypt
  available_hashtypes.append(('crypt','Crypt'))
  import md5
  available_hashtypes.append(('md5','MD5'))
  available_hashtypes.append(('smd5','salted MD5'))
  import sha
  available_hashtypes.append(('sha','SHA-1'))
  available_hashtypes.append(('ssha','salted SHA-1'))
except ImportError:
  pass


########################################################################
# Change userpassword
########################################################################

def w2l_Passwd(outf,command,form,ldap_session):

  ldap_host,ldap_binddn,ldap_bindpasswd,ldap_obj,ldap_defaults,ldap_dn = ldap_session

  if not ldap_binddn:
    ldap_binddn = ldap_dn

  if ldap_dn and \
     ('passwd_newpasswd' in form.inputkeys):

    try:
      ldap_obj.bind_s(ldap_binddn,ldap_bindpasswd,ldap.AUTH_SIMPLE)
    except ldap.INVALID_CREDENTIALS:
      raise w2lcore.ErrorExitClass(ldap_session,'Wrong password for binding as <strong>%s</strong> to %s.' % (w2lcore.utf2display(form.accept_charset,ldap_binddn),ldap_host))
    except ldap.INAPPROPRIATE_AUTH:
      raise w2lcore.ErrorExitClass(ldap_session,'Wrong password for binding as <strong>%s</strong> to %s.' % (w2lcore.utf2display(form.accept_charset,ldap_binddn),ldap_host))
    except:
      exc_obj,exc_value,exc_traceback = sys.exc_info()
      raise w2lcore.ErrorExitClass(ldap_session,'<p>Unhandled exception during binding as <strong>%s</strong> to %s:</p>%s, %s.' % (w2lcore.utf2display(form.accept_charset,ldap_binddn),ldap_host,exc_obj,exc_value))

    if len(form.field['passwd_newpasswd'][0].content)!=2:
      raise w2lcore.ErrorExitClass(ldap_session,'Repeat password!')

    if form.field['passwd_newpasswd'][0].content[0]!=form.field['passwd_newpasswd'][0].content[1]:
      raise w2lcore.ErrorExitClass(ldap_session,'New passwords do not match!')

    if not form.field['passwd_newpasswd'][0].content[0]:
      raise w2lcore.ErrorExitClass(ldap_session,'Empty passwords not allowed!')

    passwdinput = form.field['passwd_newpasswd'][0].content[0]

    if 'passwd_hash' in form.inputkeys:
      passwd_hash = string.lower(form.field['passwd_hash'][0].content)
      if passwd_hash in ['crypt','smd5','ssha']:
        crypt_salt = ''
        for i in range(2):
  	  crypt_salt = crypt_salt+charset.crypt_alphabet[random.randint(0,charset.crypt_alphabet_len-1)]
      else:
	crypt_salt = ''
      if passwd_hash=='crypt':
	new_passwd = '{crypt}%s' % crypt.crypt(form.field['passwd_newpasswd'][0].content[0],crypt_salt)
      elif passwd_hash in ['md5','smd5']:
	new_passwd = string.strip('{%s}%s' % (passwd_hash,base64.encodestring(md5.new(passwdinput+crypt_salt).digest()+crypt_salt)))
      elif passwd_hash in ['sha','ssha']:
	new_passwd = string.strip('{%s}%s' % (passwd_hash,base64.encodestring(sha.new(passwdinput+crypt_salt).digest()+crypt_salt)))
    else:
      new_passwd = passwdinput

    modlist = [(ldap.MOD_REPLACE,'userPassword',new_passwd)]

    # Passwort aendern
    try:
      ldap_obj.modify_s(ldap_dn,modlist)
    except ldap.NO_SUCH_OBJECT:
      raise w2lcore.ErrorExitClass(ldap_session,'No entry <strong>%s</strong>.' % w2lcore.utf2display(form.accept_charset,ldap_dn))
    except ldap.INSUFFICIENT_ACCESS:
      raise w2lcore.ErrorExitClass(ldap_session,'You are not allowed to modify entry <strong>%s</strong>.' % w2lcore.utf2display(form.accept_charset,ldap_dn))
    except:
      exc_obj,exc_value,exc_traceback = sys.exc_info()
      raise w2lcore.ErrorExitClass(ldap_session,'Unhandled exception: %s, %s.' % (exc_obj,exc_value))

    w2lcore.PrintHeader(outf,'Changed password',form.accept_charset)
    outf.write('<DIV ID=MessageDiv>')
    if ldap_dn==ldap_binddn:
      w2lcore.CommandTable(outf,[
	w2lcore.CommandButton(form,'login','Login as','','dummy',ldap_host,'',
	              extrastr='<INPUT TYPE=HIDDEN NAME="login_binddn" VALUE="%s">' % (w2lcore.utf2display(form.accept_charset,ldap_binddn)))
      ],div_id='ContextMenuDiv',table_id='ContextMenuTable')
      outf.write('<p>A new login is required!</p>')
    else:
      w2lcore.MainMenuTable(outf,form,ldap_session)
    outf.write('<p>Changed password of entry <strong>%s</strong>.</p></DIV>' % w2lcore.utf2display(form.accept_charset,ldap_dn))
    w2lcore.PrintFooter(outf)

  else:

    if ldap_defaults.has_key('passwd_hashtypes'):
      # The set of hash types are restricted by local configuration
      config_hashtypes = map(string.lower,ldap_defaults['passwd_hashtypes'])
      default_hashtypes = []
      for hash_type in available_hashtypes:
	if hash_type[0] in config_hashtypes:
          default_hashtypes.append(hash_type)
      form.field['passwd_hash'][0].options = default_hashtypes

    # Leeres Passworteingabeformular ausgeben
    w2lcore.PrintHeader(outf,'Change password',form.accept_charset)
    w2lcore.MainMenuTable(outf,form,ldap_session)
    outf.write("""<div id=InputDiv>
<h1>Change password</h1>
<form
  action="%s/passwd"
  method="%s"
  enctype="application/x-www-form-urlencoded"
  accept-charset="%s"
>
  <input type=hidden name="ldap_host"       value="%s">
  <input type=hidden name="ldap_binddn"     value="%s">
  <table summary="Password input form">
  <tr>
    <td>Change password for</td>
    <td><INPUT NAME="ldap_dn" MAXLENGTH=200 SIZE=30 VALUE="%s"> on %s</td>
  </tr>
  <tr>
    <td>Password for <strong>%s</strong></td>
    <td><input type="PASSWORD" NAME="ldap_bindpasswd" maxlength=30 size=15 value=""></td>
  </tr>
  <tr>
    <td>New password</td>
    <td><input type="PASSWORD" NAME="passwd_newpasswd" maxlength=30 size=30 value=""></td>
  </tr>
  <tr>
    <td>repeat new password</td>
    <td><input type="PASSWORD" NAME="passwd_newpasswd" maxlength=30 size=30 value=""></td>
  </tr>
  <tr>
    <td>Password encryption method</td>
    <td>%s</td>
  </tr>
</table>
<input type="submit" value="Change Password">
<input type="reset" value="Reset">
</form>
</div>
""" % (
    form.script_name,
    web2ldapcnf.script_method,
    form.accept_charset,
    ldap_host,
    w2lcore.utf2display(form.accept_charset,ldap_binddn),
    w2lcore.utf2display(form.accept_charset,ldap_dn),
    ldap_host,
    w2lcore.utf2display(form.accept_charset,ldap_binddn),
    form.field['passwd_hash'][0].inputfield()
  )
)

    w2lcore.PrintFooter(outf)


