########################################################################
# web2ldap
# (c) by Michael Stroeder <michael@stroeder.com>
# Distributed under GNU Public License (GPL)
# web-based LDAP Client, see http://www.web2ldap.de for details
########################################################################

import sys, string, time, \
       msbase, cgiforms, charset, ldap, ldapbase, w2lcore, web2ldapcnf

from web2ldapcnf import max_searchparams, script_method
from msbase import intersection

##############################################################################
# Search for entries and output results as table, pretty-printable output
# or LDIF formatted
##############################################################################

def w2l_Search(outf,command,form,ldap_session):

  ldap_host,ldap_binddn,ldap_bindpasswd,ldap_obj,ldap_defaults,ldap_dn = ldap_session

  # Suchparameter sind alle getrennt angegeben
  if 'search_output' in form.inputkeys:
    search_output = form.field['search_output'][0].content
  else:
    search_output = 'table'

  # Suchparameter sind alle getrennt angegeben
  if 'ldap_dn' in form.inputkeys:
    ldap_dn = form.field['ldap_dn'][0].content
  else:
    ldap_dn = ''

  # Suchparameter sind alle getrennt angegeben
  if 'search_scope' in form.inputkeys:
    search_scope_str = form.field['search_scope'][0].content
  else:
    search_scope_str = 'sub'

  # LDAP-Filter zusammenbauen
  if 'search_filterstr' in form.inputkeys:
    search_filterstr = form.field['search_filterstr'][0].content
  else:
    if 'search_mode' in form.inputkeys:
      search_mode = form.field['search_mode'][0].content
    else:
      search_mode = '(&%s)'
    search_filter = []
    if ('search_option' in form.inputkeys) and \
       ('search_attr' in form.inputkeys) and \
       ('search_string' in form.inputkeys) and \
       (
         len(form.field['search_option'][0].content)== \
	 len(form.field['search_attr'][0].content)== \
	 len(form.field['search_string'][0].content) \
       ):
      for i in range(len(form.field['search_option'][0].content)):
	if form.field['search_string'][0].content[i]:
	  search_filter.append(
	    form.field['search_option'][0].content[i] % (
    		string.lower(form.field['search_attr'][0].content[i]), \
		form.field['search_string'][0].content[i]
	    )
	  )
    else:
      raise w2lcore.ErrorExitClass(ldap_session,'Invalid or empty search form data.')
    # Teilsuchen werden alle und-verknuepft
    search_filterstr = search_mode % string.join(search_filter,'')
  search_filterstr = w2lcore.input2utf(form.accept_charset,search_filterstr)

  if 'search_resminindex' in form.inputkeys:
    search_resminindex = int(form.field['search_resminindex'][0].content)
  else:
    search_resminindex = 0
  if 'search_resnumber' in form.inputkeys:
    search_resnumber = int(form.field['search_resnumber'][0].content)
  else:
    search_resnumber = 10

  if not ldap_dn:
    ldap_dn = charset.iso2utf(ldap_defaults.get('basedn',''))
    ldap_session = ldap_host,ldap_binddn,ldap_bindpasswd,ldap_obj,ldap_defaults,ldap_dn

  search_scope = ldapbase.ldap_searchscope.get(search_scope_str,ldap.SCOPE_SUBTREE)

  if search_output=='print':

    print_template = ldap_defaults.get('print_template','')
    if not print_template:
      raise w2lcore.ErrorExitClass(ldap_session,'Print template empty.')
    print_cols = ldap_defaults.get('print_cols','')
    read_attrs = msbase.GrabKeys(print_template).keys
    read_attrs.append('objectclass')

  elif search_output=='table':

    read_attrs = []

    search_tdtemplate = ldap_defaults.get('search_tdtemplate',{})
    if search_tdtemplate:
      read_attrs = []
      for k in search_tdtemplate.keys():
        for i in msbase.GrabKeys(search_tdtemplate[k]).keys:
	  if not i in read_attrs:
	    read_attrs.append(i)

    search_tablistattrs = ldap_defaults.get('search_tablistattrs',[])
    if search_tablistattrs:
      read_attrs.extend(search_tablistattrs)

    read_attrs.append('objectclass')

  elif search_output in ['ldif','ldif1','dsml']:
    # read all attributes
    read_attrs = []

  # Suche ausfhren
  try:
    ldap_msgid = ldap_obj.search(ldap_dn,search_scope,search_filterstr,read_attrs,0)
  except ldap.NO_SUCH_OBJECT:
    result_dnlist = []
  except ldap.FILTER_ERROR:
    raise w2lcore.ErrorExitClass(ldap_session,'Bad search filter &quot;%s&quot;.' % w2lcore.utf2display(form.accept_charset,search_filterstr))
  except ldap.SIZELIMIT_EXCEEDED:
    raise w2lcore.ErrorExitClass(ldap_session,'Sizelimit exceeded. Please refine search.')
  except:
    exc_obj,exc_value,exc_traceback = sys.exc_info()
    raise w2lcore.ErrorExitClass(ldap_session,'Unhandled exception: %s, %s.' % (exc_obj,exc_value))

  #ldapurl    = scheme "://" [hostport] ["/"
  #                 [dn ["?" [attributes] ["?" [scope]
  #                 ["?" [filter] ["?" extensions]]]]]]
  # Build LDAP URL of search query according to RFC2255
  search_ldap_url = 'ldap://%s/%s?%s?%s?%s' % (
    ldap_host,
    ldap_dn,
    string.join(read_attrs,','),
    search_scope_str,
    search_filterstr,
  )

  if search_output=='table':

    search_maxhits = search_resminindex+search_resnumber
    result_dnlist = []

    result_type,result_data = ldap_obj.result(ldap_msgid,0)
    if not result_type in ['RES_SEARCH_ENTRY','RES_SEARCH_RESULT']:
      ldap_obj.abandon(ldap_msgid)
      raise w2lcore.ErrorExitClass(ldap_session,'Wrong result type: "%s"' % (result_type))

    mailtolist = []
    resind = 0
    while result_data and \
          (search_maxhits==0 or (resind<search_maxhits)):
      if resind>=search_resminindex:
	for dn,data in result_data:
	  if data.has_key('mail'):
            mailtolist.extend(data['mail'])
        result_dnlist.extend(result_data)
      result_type,result_data = ldap_obj.result(ldap_msgid,0)
      resind = resind+1

    if result_data:
      # Only partial results retrieved, undisplayed data available
      ldap_obj.abandon(ldap_msgid)

    result_dnlist.sort()

    # Output search results as table with buttons
    w2lcore.PrintHeader(outf,'Search Results',form.accept_charset)
    w2lcore.MainMenuTable(outf,form,ldap_session)

    if len(result_dnlist):

      ContextMenuList = []

      # There's still data left to be read
      if result_data or search_resminindex:

        prev_resminindex = max(0,search_resminindex-search_resnumber)

        if search_resminindex+search_resnumber<=resind:
          ContextMenuList.append(
	    w2lcore.CommandButton(
	      form,
	      'search',
	      '-&gt;&gt;',
	      w2lcore.utf2display(form.accept_charset,ldap_binddn),
	      w2lcore.utf2display(form.accept_charset,ldap_bindpasswd),
	      ldap_host,
	      ldap_dn,
	      extrastr="""
	      <input type=hidden name="search_filterstr" value="%s">
	      <input type=hidden name="search_resminindex" value="%d">
	      <input type=hidden name="search_resnumber" value="%d">
	      <input type=hidden name="search_scope" value="%s">""" % (
	        w2lcore.utf2display(form.accept_charset,search_filterstr),
		search_resminindex+search_resnumber,search_resnumber,
		search_scope_str
              )
            )
	  )

        if search_resminindex:
          ContextMenuList.append(
	    w2lcore.CommandButton(
	      form,
	      'search',
	      '&lt;&lt;-',
	      w2lcore.utf2display(form.accept_charset,ldap_binddn),
	      w2lcore.utf2display(form.accept_charset,ldap_bindpasswd),
	      ldap_host,
	      ldap_dn,
	      extrastr="""
	      <input type=hidden name="search_filterstr" value="%s">
	      <input type=hidden name="search_resminindex" value="%d">
	      <input type=hidden name="search_resnumber" value="%d">
	      <input type=hidden name="search_scope" value="%s">""" % (
	        w2lcore.utf2display(form.accept_charset,search_filterstr),
		prev_resminindex,search_resnumber,
		search_scope_str
              )
            )
	  )

	outf.write(
          """
	  <p>
	    Results %d - %d of %s search with filter &quot;%s&quot;.
	  </p>
	  """ % (
	    search_resminindex+1,
	    resind,
	    search_scope_str,
	    w2lcore.utf2display(form.accept_charset,search_filterstr)
	  )
	)

      else:
	outf.write(
          '<p>%s search with filter &quot;%s&quot; found %d entries.</p>' % (
	    search_scope_str,
	    w2lcore.utf2display(form.accept_charset,search_filterstr),
	    len(result_dnlist)
	  )
	)

      ContextMenuList.extend([
	w2lcore.CommandButton(
	  form,
	  'searchform',
	  'Refine Filter',
	  w2lcore.utf2display(form.accept_charset,ldap_binddn),
	  w2lcore.utf2display(form.accept_charset,ldap_bindpasswd),
	  ldap_host,
	  ldap_dn,
	  extrastr='<input type=hidden name="searchform_mode" value="exp"><input type=hidden name="search_filterstr" VALUE="%s"><input type=hidden name="search_scope" value="%s">' % (w2lcore.utf2display(form.accept_charset,search_filterstr),search_scope_str)
        ),
	w2lcore.CommandButton(
	  form,
	  'search',
	  'Other format',
	  w2lcore.utf2display(form.accept_charset,ldap_binddn),
	  w2lcore.utf2display(form.accept_charset,ldap_bindpasswd),
	  ldap_host,w2lcore.utf2display(form.accept_charset,ldap_dn),
          extrastr="""
	  <input type=hidden name="search_scope" value="%s">
	  <input type=hidden name="search_filterstr" value="%s">
	  <select name="search_output">
	    <option value="print" selected>Printable</option>
	    <option value="ldif">LDIF (Umich style)</option>
	    <option value="ldif1">LDIF (Version 1)</option>
	    <option value="dsml">DSML</option>
	  </select>
	  """ % (search_scope_str,w2lcore.utf2display(form.accept_charset,search_filterstr)),
	  target='_altoutput'
	)
      ])

      w2lcore.CommandTable(outf,ContextMenuList,div_id='ContextMenuDiv',table_id='ContextMenuTable')

      outf.write('<p><A HREF="%s" TARGET="_ldapurl">Search by LDAP URL</A>' % (search_ldap_url))
      if mailtolist:
	outf.write(
          '- <A HREF="mailto:%s?cc=%s">Mail to all Cc:-ed</A>- <A HREF="mailto:?bcc=%s">Mail to all Bcc:-ed</A>' % (
	    mailtolist[0],
	    string.join(mailtolist[1:],','),
	    string.join(mailtolist,',')
  	  )
  	)
      outf.write('</p>')

      search_tablistattrs_len = len(search_tablistattrs)

      outf.write('<table id=SearchResultTable summary="Table of search results" WIDTH="100%%" CELLPADDING=0 CELLSPACING=0 BORDER RULES=rows>\n')

      ldap_binddn_html4 = w2lcore.utf2display(form.accept_charset,ldap_binddn)
      ldap_bindpasswd_html4 = w2lcore.utf2display(form.accept_charset,ldap_bindpasswd)

      for entry in result_dnlist:

	dn = entry[0]
	dn_html4 = w2lcore.utf2display(form.accept_charset,dn)
        objectclasses = entry[1].get('objectclass',[])
	tdtemplate_oc = intersection(objectclasses,search_tdtemplate.keys())

        if tdtemplate_oc:

          # Print entry with the help of pre-defined templates
	  tableentry = msbase.DefaultDict('')
	  attr_list=entry[1].keys()
	  for attr in attr_list:
	    tableentry[attr] = []
	    for value in entry[1][attr]:
  	      tableentry[attr].append(w2lcore.DataStr(form,ldap_session,value))
	    tableentry[attr] = string.join(tableentry[attr],', ')
          tdlist = []
	  for oc in tdtemplate_oc:
	    tdlist.append(search_tdtemplate[oc] % tableentry)
	  tableentrystr='<td id=SearchResultItem colspan=%d>%s</td>\n' % (search_tablistattrs_len,string.join(tdlist,'<br>\n'))

	elif search_tablistattrs and entry[1].has_key(search_tablistattrs[0]):
          # Print values of attributes listed in search_tablistattrs
	  tableentry=[]
	  attr_list=entry[1].keys()
	  for attr in search_tablistattrs:
            datalist = entry[1].get(attr,[])
            if datalist:
              itemlist = []
	      for item in datalist:
	        itemlist.append(w2lcore.DataStr(form,ldap_session,item))
              tableentry.append(string.join(itemlist,'<br>'))
	    else:
	      tableentry.append('&nbsp;')
          tableentrystr='<td id=SearchResultItem>%s</td>' % string.join(tableentry,'</td><td id=SearchResultItem>')

	else:
          # Print DN
	  tableentrystr='<td id=SearchResultItem colspan=%d>%s</td>' % (search_tablistattrs_len,w2lcore.DataStr(form,ldap_session,dn))

	outf.write('<TR><TD WIDTH="5%%">%s</td><TD WIDTH="5%%">%s</td>%s</TR>' % (
          w2lcore.CommandButton(
	    form,
	    'search',
	    'Go down',
	    ldap_binddn_html4,
	    ldap_bindpasswd_html4,
	    ldap_host,
	    dn_html4,
	    extrastr='<input type=hidden name="search_scope" VALUE="one"><input type=hidden name="search_filterstr" VALUE="(objectclass=*)">'
          ),
          w2lcore.CommandButton(
	    form,
	    'read',
	    'Read',
	    ldap_binddn_html4,
	    ldap_bindpasswd_html4,
	    ldap_host,
	    dn_html4
	  ),
	  tableentrystr
	)
      )
      outf.write('</table>')

    else:

      w2lcore.CommandTable(outf,[
	w2lcore.CommandButton(
	  form,
	  'searchform',
	  'Refine Filter',
	  w2lcore.utf2display(form.accept_charset,ldap_binddn),
	  w2lcore.utf2display(form.accept_charset,ldap_bindpasswd),
	  ldap_host,
	  w2lcore.utf2display(form.accept_charset,ldap_dn),
	  extrastr='<input type=hidden name="searchform_mode" VALUE="exp"><input type=hidden name="search_filterstr" VALUE="%s"><input type=hidden name="search_scope" VALUE="%s">' % (w2lcore.utf2display(form.accept_charset,search_filterstr),search_scope_str)
        )
      ],div_id='ContextMenuDiv',table_id='ContextMenuTable')
      outf.write('<div id=MessageDiv><p>No Entries found with %s search with filter &quot;%s&quot;.</p></div>' % (search_scope_str,w2lcore.utf2display(form.accept_charset,search_filterstr)))

    w2lcore.PrintFooter(outf)

  elif search_output=='print':

    result_type,result_dnlist = ldap_obj.result(ldap_msgid,1)
    if result_type != 'RES_SEARCH_RESULT':
      raise w2lcore.ErrorExitClass(ldap_session,'Wrong result type: "%s"' % (result_type))

    result_dnlist.sort()

    if not result_dnlist:
      raise w2lcore.ErrorExitClass(ldap_session,'No search results.')

    # Output search results as pretty-printable table without buttons
    w2lcore.PrintHeader(outf,'Search Results',form.accept_charset)

    table=[]
    for entry in result_dnlist:
      tableentry = msbase.DefaultDict('')
      attr_list=entry[1].keys()
      for attr in attr_list:
        # Output HTML-4 character entities because the output
	# might be saved to a HTML file without
	# Content-type/charset header.
	tableentry[attr] = string.join(w2lcore.utf2display('html4',entry[1][attr]),', ')
      table.append(print_template % tableentry)

    outf.write("""
<table
  id=PrintTable
  summary="Table with search results formatted for printing"
  width="100%%" cellpadding="3%%" cellspacing=0 border=1 rules=rows>
    <colgroup span=%d>
    """ % (print_cols))
    tdwidth=100/print_cols
    for i in range(print_cols):
      outf.write('<col width="%d%%">\n' % (tdwidth))
    outf.write('</colgroup>\n')
    for i in range(len(table)):
      if i%print_cols==0:
        outf.write('<tr>')
      outf.write('<td width="%d%%" align=left valign=top>%s</td>' % (tdwidth,table[i]))
      if i%print_cols==print_cols-1:
        outf.write('</tr>\n')
    if len(table)%print_cols!=print_cols-1:
      outf.write('</tr>\n')

    outf.write('</table>\n')

    w2lcore.PrintFooter(outf)


  elif search_output in ['ldif','ldif1']:

    import ldif

    result_type,result_dnlist = ldap_obj.result(ldap_msgid,0)
    if result_type !='RES_SEARCH_ENTRY':
      ldap_obj.abandon(ldap_msgid)
      raise w2lcore.ErrorExitClass(ldap_session,'Wrong result type: "%s"' % (result_type))

    # Output search results as LDIF
    w2lcore.HTTPHeader(outf,'text/plain',charset='US-ASCII')
    if search_output=='ldif1':
      outf.write("""########################################################################
# LDIF export by web2ldap %s, see http://www.web2ldap.de
# Date and time: %s
# Bind-DN: %s
# LDAP-URL of search:
# %s
########################################################################
version: 1

""" % (w2lcore.Version,time.strftime('%A, %Y-%m-%d %H:%M:%S GMT',time.gmtime(time.time()+web2ldapcnf.sec_expire)),ldap_binddn,search_ldap_url))
    while result_dnlist:
      for entry in result_dnlist:
	outf.write('%s\n' % ldif.CreateLDIF(entry[0],entry[1],w2lcore.ldap_binaryattrkeys))
      result_type,result_dnlist = ldap_obj.result(ldap_msgid,0)


  elif search_output=='dsml':

    import dsml

    result_type,result_dnlist = ldap_obj.result(ldap_msgid,0)
    if result_type !='RES_SEARCH_ENTRY':
      ldap_obj.abandon(ldap_msgid)
      raise w2lcore.ErrorExitClass(ldap_session,'Wrong result type: "%s"' % (result_type))

    # Output search results as DSML in a rather primitive way.
    # (level 1 producer)
    w2lcore.HTTPHeader(outf,'text/xml',charset='UTF-8')
    outf.write("""<dsml:dsml xmlns:dsml="http://www.dsml.org/DSML">
  <!-- directory entries found by %s -->
  <dsml:directory-entries>
""" % (search_ldap_url))

    while result_dnlist:
      for entry in result_dnlist:
	dsml.CreateDSML(outf,entry[0],entry[1],w2lcore.ldap_binaryattrkeys)
      result_type,result_dnlist = ldap_obj.result(ldap_msgid,0)

    outf.write("""  </dsml:directory-entries>
</dsml:dsml>
""")

