########################################################################
# web2ldap
# (c) by Michael Stroeder <michael@stroeder.com>
# Distributed under GNU Public License (GPL)
# web-based LDAP Client, see http://www.web2ldap.de for details
########################################################################

import string, cgiforms, charset, w2lcore, msbase, web2ldapcnf

########################################################################
# Output a pretty search form
########################################################################

def w2l_SearchForm(outf,command,form,ldap_session):

  ldap_host,ldap_binddn,ldap_bindpasswd,ldap_obj,ldap_defaults,ldap_dn = ldap_session

  if not ldap_dn:
    ldap_dn = charset.iso2utf(ldap_defaults.get('basedn',''))
    ldap_session = ldap_host,ldap_binddn,ldap_bindpasswd,ldap_obj,ldap_defaults,ldap_dn

  if 'search_scope' in form.inputkeys:
    form.field['search_scope'][0].setdefault(form.field['search_scope'][0].content)

  if 'searchform_mode' in form.inputkeys:
    searchform_mode = form.field['searchform_mode'][0].content
  else:
    searchform_mode = 'base'

  w2lcore.PrintHeader(outf,'',form.accept_charset)
  w2lcore.MainMenuTable(outf,form,ldap_session)

  if searchform_mode == 'base':

    # print base search form with fixed input fields

    searchform_template_filename = w2lcore.GetVariantFilename(
      ldap_defaults.get('searchform_template',''),
      map(string.strip,string.split(form.env.get('HTTP_ACCEPT_LANGUAGE',''),','))
    )

    if searchform_template_filename:
      try:
  	template_str = open(searchform_template_filename,'r').read()
	outf.write('<div id=InputDiv><h1>Base Search Form</h1>')
	outf.write(
	  '<form action="%s/search" method="%s" enctype="application/x-www-form-urlencoded" accept-charset="%s">\n%s\n' % (
            form.script_name,
	    web2ldapcnf.script_method,
	    form.accept_charset,
            template_str
	  )
	)

      except IOError:
	outf.write('<div id=MessageDiv><p>I/O error during reading template file.</p></div>')
    else:
      outf.write('<div id=MessageDiv><p>Template file not found.</p></div>')


  elif searchform_mode == 'exp':

    # print expert search form for using LDAP filters
    outf.write('<div id=InputDiv><h1>Expert Search Form</h1>')
    search_filterstr = form.field.get('search_filterstr',[])
    search_filterstr = search_filterstr[0].content
    outf.write("""<form action="%s/search" method="%s" enctype="application/x-www-form-urlencoded" accept-charset="%s">
             LDAP filter string<BR>
	     <input name="search_filterstr" maxlength="120" size="60" value="%s">
	     """ % (form.script_name,web2ldapcnf.script_method,form.accept_charset,search_filterstr))

  elif searchform_mode == 'adv':

    # print advanced search form
    outf.write('<div id=InputDiv><h1>Advanced Search Form</h1>')

    if ('search_params' in form.inputkeys) and \
       form.field['search_params'][0].content:
      # Gewuenschte Anzahl der Suchparameter wurde angegeben
      rows = string.atoi(form.field['search_params'][0].content)
      if rows<1:                rows=1
      if rows>web2ldapcnf.max_searchparams: rows=web2ldapcnf.max_searchparams
    else:
      rows = 1

    # More und Fewer-Buttons ausgeben
    outf.write('<table><tr>')
    if rows<web2ldapcnf.max_searchparams:
      outf.write(
        '<td>%s</td>' % w2lcore.CommandButton(
	  form,'searchform','More',
	  w2lcore.utf2display(form.accept_charset,ldap_binddn),
	  w2lcore.utf2display(form.accept_charset,ldap_bindpasswd),
	  ldap_host,
	  extrastr='<input type=hidden name="searchform_mode" value="adv"><input type=hidden name="search_params" value="%d">' % (rows+1)
        )
      )
    if rows>1:
      outf.write('<td>%s</td>' % w2lcore.CommandButton(
          form,'searchform','Fewer',
	  w2lcore.utf2display(form.accept_charset,ldap_binddn),
	  w2lcore.utf2display(form.accept_charset,ldap_bindpasswd),
	  ldap_host,
	  extrastr='<input type=hidden name="searchform_mode" value="adv"><input type=hidden name="search_params" value="%d">' % (rows-1)
        )
      )

    outf.write('<td>search parameters</td></tr></table>')

    # Eigentliches Suchformular ausgeben
    outf.write("""<form action="%s/search" method="%s" enctype="application/x-www-form-urlencoded" accept-charset="%s">
Match
<select name="search_mode">
  <option value="(&amp;%%s)" selected>all
  <option value="(|%%s)">any
</select>
of the following.<br>""" % (form.script_name,web2ldapcnf.script_method,form.accept_charset))

    # Output a row of the search form
    for i in range(rows):

      # Output of attribute select field
      outf.write('<select name="search_attr" size=1>')
      for ind in range(len(web2ldapcnf.search_attrs)):
	outf.write('<option value="%s"%s>%s\n' % (
	    web2ldapcnf.search_attrs[ind],
	    ' selected'*(ind==i),
	    w2lcore.ldap_knownattr.get(web2ldapcnf.search_attrs[ind],web2ldapcnf.search_attrs[ind])
	  )
	)
      outf.write('</select>\n')

      # Output of search option select field
      outf.write('<select name="search_option" size=1>')
      for option in w2lcore.search_option.keys():
	outf.write('  <option value="%s" %s>%s' % (
	    option,
	    ' selected'*(option==r'(%s=*%s*)'),
	    w2lcore.search_option[option]
	  )
	)
      outf.write('</select>\n')

      # Output of search string input field
      outf.write('<input name="search_string" maxlength="60" size="30" value=""><br>\n')


  # Output of form part for all kind of search forms
  outf.write("""
<p>
 Search root
 <input name="ldap_dn" maxlength="200" size="30" value="%s">
 Scope %s
 Results per page %s
</p>
 <input type=hidden name="ldap_binddn"     value="%s">
 <input type=hidden name="ldap_bindpasswd" value="%s">
 <input type=hidden name="ldap_host"       value="%s">
 <input type="submit" value="Search">
 <input type="reset" value="Reset">
 </form>
</div>
""" % (
    ldap_dn,
    form.field['search_scope'][0].inputfield(),
    form.field['search_resnumber'][0].inputfield(),
    ldap_binddn,
    ldap_bindpasswd,
    ldap_host
  ))

  w2lcore.PrintFooter(outf)

